from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._resource import Resource
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.volumes._models.lifecycle import Lifecycle, LifecycleParam
from databricks.bundles.volumes._models.volume_grant import (
    VolumeGrant,
    VolumeGrantParam,
)
from databricks.bundles.volumes._models.volume_type import VolumeType, VolumeTypeParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Volume(Resource):
    """"""

    catalog_name: VariableOr[str]
    """
    The name of the catalog where the schema and the volume are
    """

    name: VariableOr[str]
    """
    The name of the volume
    """

    schema_name: VariableOr[str]
    """
    The name of the schema where the volume is
    """

    comment: VariableOrOptional[str] = None
    """
    The comment attached to the volume
    """

    grants: VariableOrList[VolumeGrant] = field(default_factory=list)

    lifecycle: VariableOrOptional[Lifecycle] = None
    """
    Lifecycle is a struct that contains the lifecycle settings for a resource. It controls the behavior of the resource when it is deployed or destroyed.
    """

    storage_location: VariableOrOptional[str] = None
    """
    The storage location on the cloud
    """

    volume_type: VariableOrOptional[VolumeType] = None

    @classmethod
    def from_dict(cls, value: "VolumeDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "VolumeDict":
        return _transform_to_json_value(self)  # type:ignore


class VolumeDict(TypedDict, total=False):
    """"""

    catalog_name: VariableOr[str]
    """
    The name of the catalog where the schema and the volume are
    """

    name: VariableOr[str]
    """
    The name of the volume
    """

    schema_name: VariableOr[str]
    """
    The name of the schema where the volume is
    """

    comment: VariableOrOptional[str]
    """
    The comment attached to the volume
    """

    grants: VariableOrList[VolumeGrantParam]

    lifecycle: VariableOrOptional[LifecycleParam]
    """
    Lifecycle is a struct that contains the lifecycle settings for a resource. It controls the behavior of the resource when it is deployed or destroyed.
    """

    storage_location: VariableOrOptional[str]
    """
    The storage location on the cloud
    """

    volume_type: VariableOrOptional[VolumeTypeParam]


VolumeParam = VolumeDict | Volume



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.pipelines._models.postgres_slot_config import (
    PostgresSlotConfig,
    PostgresSlotConfigParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PostgresCatalogConfig:
    """
    PG-specific catalog-level configuration parameters
    """

    slot_config: VariableOrOptional[PostgresSlotConfig] = None
    """
    Optional. The Postgres slot configuration to use for logical replication
    """

    @classmethod
    def from_dict(cls, value: "PostgresCatalogConfigDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PostgresCatalogConfigDict":
        return _transform_to_json_value(self)  # type:ignore


class PostgresCatalogConfigDict(TypedDict, total=False):
    """"""

    slot_config: VariableOrOptional[PostgresSlotConfigParam]
    """
    Optional. The Postgres slot configuration to use for logical replication
    """


PostgresCatalogConfigParam = PostgresCatalogConfigDict | PostgresCatalogConfig



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr
from databricks.bundles.jobs._models.clients_types import (
    ClientsTypes,
    ClientsTypesParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class WorkloadType:
    """
    Cluster Attributes showing for clusters workload types.
    """

    clients: VariableOr[ClientsTypes]
    """
    defined what type of clients can use the cluster. E.g. Notebooks, Jobs
    """

    @classmethod
    def from_dict(cls, value: "WorkloadTypeDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "WorkloadTypeDict":
        return _transform_to_json_value(self)  # type:ignore


class WorkloadTypeDict(TypedDict, total=False):
    """"""

    clients: VariableOr[ClientsTypesParam]
    """
    defined what type of clients can use the cluster. E.g. Notebooks, Jobs
    """


WorkloadTypeParam = WorkloadTypeDict | WorkloadType



from enum import Enum
from typing import Literal


class StorageMode(Enum):
    DIRECT_QUERY = "DIRECT_QUERY"
    IMPORT = "IMPORT"
    DUAL = "DUAL"


StorageModeParam = Literal["DIRECT_QUERY", "IMPORT", "DUAL"] | StorageMode



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOrDict,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.pipelines._models.ingestion_pipeline_definition_workday_report_parameters_query_key_value import (
    IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValue,
    IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class IngestionPipelineDefinitionWorkdayReportParameters:
    """
    :meta private: [EXPERIMENTAL]
    """

    incremental: VariableOrOptional[bool] = None
    """
    [DEPRECATED] (Optional) Marks the report as incremental.
    This field is deprecated and should not be used. Use `parameters` instead. The incremental behavior is now
    controlled by the `parameters` field.
    """

    parameters: VariableOrDict[str] = field(default_factory=dict)
    """
    Parameters for the Workday report. Each key represents the parameter name (e.g., "start_date", "end_date"),
    and the corresponding value is a SQL-like expression used to compute the parameter value at runtime.
    Example:
    {
    "start_date": "{ coalesce(current_offset(), date(\"2025-02-01\")) }",
    "end_date": "{ current_date() - INTERVAL 1 DAY }"
    }
    """

    report_parameters: VariableOrList[
        IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValue
    ] = field(default_factory=list)
    """
    [DEPRECATED] (Optional) Additional custom parameters for Workday Report
    This field is deprecated and should not be used. Use `parameters` instead.
    """

    @classmethod
    def from_dict(
        cls, value: "IngestionPipelineDefinitionWorkdayReportParametersDict"
    ) -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "IngestionPipelineDefinitionWorkdayReportParametersDict":
        return _transform_to_json_value(self)  # type:ignore


class IngestionPipelineDefinitionWorkdayReportParametersDict(TypedDict, total=False):
    """"""

    incremental: VariableOrOptional[bool]
    """
    [DEPRECATED] (Optional) Marks the report as incremental.
    This field is deprecated and should not be used. Use `parameters` instead. The incremental behavior is now
    controlled by the `parameters` field.
    """

    parameters: VariableOrDict[str]
    """
    Parameters for the Workday report. Each key represents the parameter name (e.g., "start_date", "end_date"),
    and the corresponding value is a SQL-like expression used to compute the parameter value at runtime.
    Example:
    {
    "start_date": "{ coalesce(current_offset(), date(\"2025-02-01\")) }",
    "end_date": "{ current_date() - INTERVAL 1 DAY }"
    }
    """

    report_parameters: VariableOrList[
        IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueParam
    ]
    """
    [DEPRECATED] (Optional) Additional custom parameters for Workday Report
    This field is deprecated and should not be used. Use `parameters` instead.
    """


IngestionPipelineDefinitionWorkdayReportParametersParam = (
    IngestionPipelineDefinitionWorkdayReportParametersDict
    | IngestionPipelineDefinitionWorkdayReportParameters
)



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOrDict,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.pipelines._models.aws_attributes import (
    AwsAttributes,
    AwsAttributesParam,
)
from databricks.bundles.pipelines._models.azure_attributes import (
    AzureAttributes,
    AzureAttributesParam,
)
from databricks.bundles.pipelines._models.cluster_log_conf import (
    ClusterLogConf,
    ClusterLogConfParam,
)
from databricks.bundles.pipelines._models.gcp_attributes import (
    GcpAttributes,
    GcpAttributesParam,
)
from databricks.bundles.pipelines._models.init_script_info import (
    InitScriptInfo,
    InitScriptInfoParam,
)
from databricks.bundles.pipelines._models.pipeline_cluster_autoscale import (
    PipelineClusterAutoscale,
    PipelineClusterAutoscaleParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PipelineCluster:
    """"""

    apply_policy_default_values: VariableOrOptional[bool] = None
    """
    Note: This field won't be persisted. Only API users will check this field.
    """

    autoscale: VariableOrOptional[PipelineClusterAutoscale] = None
    """
    Parameters needed in order to automatically scale clusters up and down based on load.
    Note: autoscaling works best with DB runtime versions 3.0 or later.
    """

    aws_attributes: VariableOrOptional[AwsAttributes] = None
    """
    Attributes related to clusters running on Amazon Web Services.
    If not specified at cluster creation, a set of default values will be used.
    """

    azure_attributes: VariableOrOptional[AzureAttributes] = None
    """
    Attributes related to clusters running on Microsoft Azure.
    If not specified at cluster creation, a set of default values will be used.
    """

    cluster_log_conf: VariableOrOptional[ClusterLogConf] = None
    """
    The configuration for delivering spark logs to a long-term storage destination.
    Only dbfs destinations are supported. Only one destination can be specified
    for one cluster. If the conf is given, the logs will be delivered to the destination every
    `5 mins`. The destination of driver logs is `$destination/$clusterId/driver`, while
    the destination of executor logs is `$destination/$clusterId/executor`.
    """

    custom_tags: VariableOrDict[str] = field(default_factory=dict)
    """
    Additional tags for cluster resources. Databricks will tag all cluster resources (e.g., AWS
    instances and EBS volumes) with these tags in addition to `default_tags`. Notes:
    
    - Currently, Databricks allows at most 45 custom tags
    
    - Clusters can only reuse cloud resources if the resources' tags are a subset of the cluster tags
    """

    driver_instance_pool_id: VariableOrOptional[str] = None
    """
    The optional ID of the instance pool for the driver of the cluster belongs.
    The pool cluster uses the instance pool with id (instance_pool_id) if the driver pool is not
    assigned.
    """

    driver_node_type_id: VariableOrOptional[str] = None
    """
    The node type of the Spark driver.
    Note that this field is optional; if unset, the driver node type will be set as the same value
    as `node_type_id` defined above.
    """

    enable_local_disk_encryption: VariableOrOptional[bool] = None
    """
    Whether to enable local disk encryption for the cluster.
    """

    gcp_attributes: VariableOrOptional[GcpAttributes] = None
    """
    Attributes related to clusters running on Google Cloud Platform.
    If not specified at cluster creation, a set of default values will be used.
    """

    init_scripts: VariableOrList[InitScriptInfo] = field(default_factory=list)
    """
    The configuration for storing init scripts. Any number of destinations can be specified. The scripts are executed sequentially in the order provided. If `cluster_log_conf` is specified, init script logs are sent to `<destination>/<cluster-ID>/init_scripts`.
    """

    instance_pool_id: VariableOrOptional[str] = None
    """
    The optional ID of the instance pool to which the cluster belongs.
    """

    label: VariableOrOptional[str] = None
    """
    A label for the cluster specification, either `default` to configure the default cluster, or `maintenance` to configure the maintenance cluster. This field is optional. The default value is `default`.
    """

    node_type_id: VariableOrOptional[str] = None
    """
    This field encodes, through a single value, the resources available to each of
    the Spark nodes in this cluster. For example, the Spark nodes can be provisioned
    and optimized for memory or compute intensive workloads. A list of available node
    types can be retrieved by using the :method:clusters/listNodeTypes API call.
    """

    num_workers: VariableOrOptional[int] = None
    """
    Number of worker nodes that this cluster should have. A cluster has one Spark Driver
    and `num_workers` Executors for a total of `num_workers` + 1 Spark nodes.
    
    Note: When reading the properties of a cluster, this field reflects the desired number
    of workers rather than the actual current number of workers. For instance, if a cluster
    is resized from 5 to 10 workers, this field will immediately be updated to reflect
    the target size of 10 workers, whereas the workers listed in `spark_info` will gradually
    increase from 5 to 10 as the new nodes are provisioned.
    """

    policy_id: VariableOrOptional[str] = None
    """
    The ID of the cluster policy used to create the cluster if applicable.
    """

    spark_conf: VariableOrDict[str] = field(default_factory=dict)
    """
    An object containing a set of optional, user-specified Spark configuration key-value pairs.
    See :method:clusters/create for more details.
    """

    spark_env_vars: VariableOrDict[str] = field(default_factory=dict)
    """
    An object containing a set of optional, user-specified environment variable key-value pairs.
    Please note that key-value pair of the form (X,Y) will be exported as is (i.e.,
    `export X='Y'`) while launching the driver and workers.
    
    In order to specify an additional set of `SPARK_DAEMON_JAVA_OPTS`, we recommend appending
    them to `$SPARK_DAEMON_JAVA_OPTS` as shown in the example below. This ensures that all
    default databricks managed environmental variables are included as well.
    
    Example Spark environment variables:
    `{"SPARK_WORKER_MEMORY": "28000m", "SPARK_LOCAL_DIRS": "/local_disk0"}` or
    `{"SPARK_DAEMON_JAVA_OPTS": "$SPARK_DAEMON_JAVA_OPTS -Dspark.shuffle.service.enabled=true"}`
    """

    ssh_public_keys: VariableOrList[str] = field(default_factory=list)
    """
    SSH public key contents that will be added to each Spark node in this cluster. The
    corresponding private keys can be used to login with the user name `ubuntu` on port `2200`.
    Up to 10 keys can be specified.
    """

    @classmethod
    def from_dict(cls, value: "PipelineClusterDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PipelineClusterDict":
        return _transform_to_json_value(self)  # type:ignore


class PipelineClusterDict(TypedDict, total=False):
    """"""

    apply_policy_default_values: VariableOrOptional[bool]
    """
    Note: This field won't be persisted. Only API users will check this field.
    """

    autoscale: VariableOrOptional[PipelineClusterAutoscaleParam]
    """
    Parameters needed in order to automatically scale clusters up and down based on load.
    Note: autoscaling works best with DB runtime versions 3.0 or later.
    """

    aws_attributes: VariableOrOptional[AwsAttributesParam]
    """
    Attributes related to clusters running on Amazon Web Services.
    If not specified at cluster creation, a set of default values will be used.
    """

    azure_attributes: VariableOrOptional[AzureAttributesParam]
    """
    Attributes related to clusters running on Microsoft Azure.
    If not specified at cluster creation, a set of default values will be used.
    """

    cluster_log_conf: VariableOrOptional[ClusterLogConfParam]
    """
    The configuration for delivering spark logs to a long-term storage destination.
    Only dbfs destinations are supported. Only one destination can be specified
    for one cluster. If the conf is given, the logs will be delivered to the destination every
    `5 mins`. The destination of driver logs is `$destination/$clusterId/driver`, while
    the destination of executor logs is `$destination/$clusterId/executor`.
    """

    custom_tags: VariableOrDict[str]
    """
    Additional tags for cluster resources. Databricks will tag all cluster resources (e.g., AWS
    instances and EBS volumes) with these tags in addition to `default_tags`. Notes:
    
    - Currently, Databricks allows at most 45 custom tags
    
    - Clusters can only reuse cloud resources if the resources' tags are a subset of the cluster tags
    """

    driver_instance_pool_id: VariableOrOptional[str]
    """
    The optional ID of the instance pool for the driver of the cluster belongs.
    The pool cluster uses the instance pool with id (instance_pool_id) if the driver pool is not
    assigned.
    """

    driver_node_type_id: VariableOrOptional[str]
    """
    The node type of the Spark driver.
    Note that this field is optional; if unset, the driver node type will be set as the same value
    as `node_type_id` defined above.
    """

    enable_local_disk_encryption: VariableOrOptional[bool]
    """
    Whether to enable local disk encryption for the cluster.
    """

    gcp_attributes: VariableOrOptional[GcpAttributesParam]
    """
    Attributes related to clusters running on Google Cloud Platform.
    If not specified at cluster creation, a set of default values will be used.
    """

    init_scripts: VariableOrList[InitScriptInfoParam]
    """
    The configuration for storing init scripts. Any number of destinations can be specified. The scripts are executed sequentially in the order provided. If `cluster_log_conf` is specified, init script logs are sent to `<destination>/<cluster-ID>/init_scripts`.
    """

    instance_pool_id: VariableOrOptional[str]
    """
    The optional ID of the instance pool to which the cluster belongs.
    """

    label: VariableOrOptional[str]
    """
    A label for the cluster specification, either `default` to configure the default cluster, or `maintenance` to configure the maintenance cluster. This field is optional. The default value is `default`.
    """

    node_type_id: VariableOrOptional[str]
    """
    This field encodes, through a single value, the resources available to each of
    the Spark nodes in this cluster. For example, the Spark nodes can be provisioned
    and optimized for memory or compute intensive workloads. A list of available node
    types can be retrieved by using the :method:clusters/listNodeTypes API call.
    """

    num_workers: VariableOrOptional[int]
    """
    Number of worker nodes that this cluster should have. A cluster has one Spark Driver
    and `num_workers` Executors for a total of `num_workers` + 1 Spark nodes.
    
    Note: When reading the properties of a cluster, this field reflects the desired number
    of workers rather than the actual current number of workers. For instance, if a cluster
    is resized from 5 to 10 workers, this field will immediately be updated to reflect
    the target size of 10 workers, whereas the workers listed in `spark_info` will gradually
    increase from 5 to 10 as the new nodes are provisioned.
    """

    policy_id: VariableOrOptional[str]
    """
    The ID of the cluster policy used to create the cluster if applicable.
    """

    spark_conf: VariableOrDict[str]
    """
    An object containing a set of optional, user-specified Spark configuration key-value pairs.
    See :method:clusters/create for more details.
    """

    spark_env_vars: VariableOrDict[str]
    """
    An object containing a set of optional, user-specified environment variable key-value pairs.
    Please note that key-value pair of the form (X,Y) will be exported as is (i.e.,
    `export X='Y'`) while launching the driver and workers.
    
    In order to specify an additional set of `SPARK_DAEMON_JAVA_OPTS`, we recommend appending
    them to `$SPARK_DAEMON_JAVA_OPTS` as shown in the example below. This ensures that all
    default databricks managed environmental variables are included as well.
    
    Example Spark environment variables:
    `{"SPARK_WORKER_MEMORY": "28000m", "SPARK_LOCAL_DIRS": "/local_disk0"}` or
    `{"SPARK_DAEMON_JAVA_OPTS": "$SPARK_DAEMON_JAVA_OPTS -Dspark.shuffle.service.enabled=true"}`
    """

    ssh_public_keys: VariableOrList[str]
    """
    SSH public key contents that will be added to each Spark node in this cluster. The
    corresponding private keys can be used to login with the user name `ubuntu` on port `2200`.
    Up to 10 keys can be specified.
    """


PipelineClusterParam = PipelineClusterDict | PipelineCluster



from enum import Enum
from typing import Literal


class PerformanceTarget(Enum):
    """
    PerformanceTarget defines how performant (lower latency) or cost efficient the execution of run on serverless compute should be.
    The performance mode on the job or pipeline should map to a performance setting that is passed to Cluster Manager
    (see cluster-common PerformanceTarget).
    """

    PERFORMANCE_OPTIMIZED = "PERFORMANCE_OPTIMIZED"
    STANDARD = "STANDARD"


PerformanceTargetParam = (
    Literal["PERFORMANCE_OPTIMIZED", "STANDARD"] | PerformanceTarget
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class ConnectionParameters:
    """
    :meta private: [EXPERIMENTAL]
    """

    source_catalog: VariableOrOptional[str] = None
    """
    :meta private: [EXPERIMENTAL]
    
    Source catalog for initial connection.
    This is necessary for schema exploration in some database systems like Oracle, and optional but nice-to-have
    in some other database systems like Postgres.
    For Oracle databases, this maps to a service name.
    """

    @classmethod
    def from_dict(cls, value: "ConnectionParametersDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "ConnectionParametersDict":
        return _transform_to_json_value(self)  # type:ignore


class ConnectionParametersDict(TypedDict, total=False):
    """"""

    source_catalog: VariableOrOptional[str]
    """
    :meta private: [EXPERIMENTAL]
    
    Source catalog for initial connection.
    This is necessary for schema exploration in some database systems like Oracle, and optional but nice-to-have
    in some other database systems like Postgres.
    For Oracle databases, this maps to a service name.
    """


ConnectionParametersParam = ConnectionParametersDict | ConnectionParameters



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrList,
    VariableOrOptional,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class MavenLibrary:
    """"""

    coordinates: VariableOr[str]
    """
    Gradle-style maven coordinates. For example: "org.jsoup:jsoup:1.7.2".
    """

    exclusions: VariableOrList[str] = field(default_factory=list)
    """
    List of dependences to exclude. For example: `["slf4j:slf4j", "*:hadoop-client"]`.
    
    Maven dependency exclusions:
    https://maven.apache.org/guides/introduction/introduction-to-optional-and-excludes-dependencies.html.
    """

    repo: VariableOrOptional[str] = None
    """
    Maven repo to install the Maven package from. If omitted, both Maven Central Repository
    and Spark Packages are searched.
    """

    @classmethod
    def from_dict(cls, value: "MavenLibraryDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "MavenLibraryDict":
        return _transform_to_json_value(self)  # type:ignore


class MavenLibraryDict(TypedDict, total=False):
    """"""

    coordinates: VariableOr[str]
    """
    Gradle-style maven coordinates. For example: "org.jsoup:jsoup:1.7.2".
    """

    exclusions: VariableOrList[str]
    """
    List of dependences to exclude. For example: `["slf4j:slf4j", "*:hadoop-client"]`.
    
    Maven dependency exclusions:
    https://maven.apache.org/guides/introduction/introduction-to-optional-and-excludes-dependencies.html.
    """

    repo: VariableOrOptional[str]
    """
    Maven repo to install the Maven package from. If omitted, both Maven Central Repository
    and Spark Packages are searched.
    """


MavenLibraryParam = MavenLibraryDict | MavenLibrary



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional
from databricks.bundles.jobs._models.job_permission_level import (
    JobPermissionLevel,
    JobPermissionLevelParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class JobPermission:
    """"""

    level: VariableOr[JobPermissionLevel]

    group_name: VariableOrOptional[str] = None

    service_principal_name: VariableOrOptional[str] = None

    user_name: VariableOrOptional[str] = None

    @classmethod
    def from_dict(cls, value: "JobPermissionDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "JobPermissionDict":
        return _transform_to_json_value(self)  # type:ignore


class JobPermissionDict(TypedDict, total=False):
    """"""

    level: VariableOr[JobPermissionLevelParam]

    group_name: VariableOrOptional[str]

    service_principal_name: VariableOrOptional[str]

    user_name: VariableOrOptional[str]


JobPermissionParam = JobPermissionDict | JobPermission



import traceback
from dataclasses import dataclass, field, replace
from enum import Enum
from io import StringIO
from typing import TYPE_CHECKING, Optional, TypeVar

from databricks.bundles.core._location import Location

_T = TypeVar("_T")

if TYPE_CHECKING:
    from typing_extensions import Self

__all__ = [
    "Diagnostic",
    "Diagnostics",
    "Severity",
]


class Severity(Enum):
    WARNING = "warning"
    ERROR = "error"


@dataclass(kw_only=True, frozen=True)
class Diagnostic:
    severity: Severity
    """
    Severity of the diagnostics item.
    """

    summary: str
    """
    Short summary of the error or warning.
    """

    detail: Optional[str] = None
    """
    Explanation of the error or warning.
    """

    path: Optional[tuple[str, ...]] = None
    """
    Path in databricks.yml where the error or warning occurred.
    """

    location: Optional[Location] = None
    """
    Source code location where the error or warning occurred.
    """

    def as_dict(self) -> dict:
        def omit_none(values: dict):
            return {key: value for key, value in values.items() if value is not None}

        if self.location:
            location = self.location.as_dict()
        else:
            location = None

        return omit_none(
            {
                "severity": self.severity.value,
                "summary": self.summary,
                "detail": self.detail,
                "path": self.path,
                "location": location,
            }
        )


@dataclass(frozen=True)
class Diagnostics:
    """
    Diagnostics is a collection of errors and warnings we print to users.

    Each item can have source location or path associated, that is reported in output to
    indicate where the error or warning occurred.
    """

    items: tuple[Diagnostic, ...] = field(default_factory=tuple, kw_only=False)

    def extend(self, diagnostics: "Self") -> "Self":
        """
        Extend items with another diagnostics. This pattern allows
        to accumulate errors and warnings.

        Example:

        .. code-block:: python

            def foo() -> Diagnostics: ...
            def bar() -> Diagnostics: ...

            diagnostics = Diagnostics()
            diagnostics = diagnostics.extend(foo())
            diagnostics = diagnostics.extend(bar())
        """

        return replace(
            self,
            items=(*self.items, *diagnostics.items),
        )

    def extend_tuple(self, pair: tuple[_T, "Self"]) -> tuple[_T, "Self"]:
        """
        Extend items with another diagnostics. This variant is useful when
        methods return a pair of value and diagnostics. This pattern allows
        to accumulate errors and warnings.

        Example:

        .. code-block:: python

            def foo() -> (int, Diagnostics): ...

            diagnostics = Diagnostics()
            value, diagnostics = diagnostics.extend_tuple(foo())
        """

        value, other_diagnostics = pair

        return value, self.extend(other_diagnostics)

    def has_error(self) -> bool:
        """
        Returns True if there is at least one error in diagnostics.
        """

        for item in self.items:
            if item.severity == Severity.ERROR:
                return True

        return False

    def has_warning(self) -> bool:
        """
        Returns True if there is at least one warning in diagnostics.
        """

        for item in self.items:
            if item.severity == Severity.WARNING:
                return True

        return False

    @classmethod
    def create_error(
        cls,
        msg: str,
        *,
        detail: Optional[str] = None,
        location: Optional[Location] = None,
        path: Optional[tuple[str, ...]] = None,
    ) -> "Self":
        """
        Create an error diagnostics.
        """

        return cls(
            items=(
                Diagnostic(
                    severity=Severity.ERROR,
                    summary=msg,
                    detail=detail,
                    location=location,
                    path=path,
                ),
            ),
        )

    @classmethod
    def create_warning(
        cls,
        msg: str,
        *,
        detail: Optional[str] = None,
        location: Optional[Location] = None,
        path: Optional[tuple[str, ...]] = None,
    ) -> "Self":
        """
        Create a warning diagnostics.
        """

        return cls(
            items=(
                Diagnostic(
                    severity=Severity.WARNING,
                    summary=msg,
                    detail=detail,
                    location=location,
                    path=path,
                ),
            )
        )

    @classmethod
    def from_exception(
        cls,
        exc: Exception,
        *,
        summary: str,
        location: Optional[Location] = None,
        path: Optional[tuple[str, ...]] = None,
        explanation: Optional[str] = None,
    ) -> "Self":
        """
        Create diagnostics from an exception.

        :param exc: exception to create diagnostics from
        :param summary: short summary of the error
        :param location: optional location in the source code where the error occurred
        :param path: optional path to relevant property in databricks.yml
        :param explanation: optional explanation to add to the details
        """

        detail_io = StringIO()
        traceback.print_exception(exc, file=detail_io)

        detail = detail_io.getvalue()
        if explanation:
            detail = f"{detail}\n\n\033[0;36mExplanation:\033[0m {explanation}"

        diagnostic = Diagnostic(
            severity=Severity.ERROR,
            summary=summary,
            location=location,
            path=path,
            detail=detail,
        )

        return cls(items=(diagnostic,))



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class JobRunAs:
    """
    Write-only setting. Specifies the user or service principal that the job runs as. If not specified, the job runs as the user who created the job.

    Either `user_name` or `service_principal_name` should be specified. If not, an error is thrown.
    """

    service_principal_name: VariableOrOptional[str] = None
    """
    The application ID of an active service principal. Setting this field requires the `servicePrincipal/user` role.
    """

    user_name: VariableOrOptional[str] = None
    """
    The email of an active workspace user. Non-admin users can only set this field to their own email.
    """

    @classmethod
    def from_dict(cls, value: "JobRunAsDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "JobRunAsDict":
        return _transform_to_json_value(self)  # type:ignore


class JobRunAsDict(TypedDict, total=False):
    """"""

    service_principal_name: VariableOrOptional[str]
    """
    The application ID of an active service principal. Setting this field requires the `servicePrincipal/user` role.
    """

    user_name: VariableOrOptional[str]
    """
    The email of an active workspace user. Non-admin users can only set this field to their own email.
    """


JobRunAsParam = JobRunAsDict | JobRunAs



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.model_trigger_configuration_condition import (
    ModelTriggerConfigurationCondition,
    ModelTriggerConfigurationConditionParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class ModelTriggerConfiguration:
    """
    :meta private: [EXPERIMENTAL]
    """

    condition: VariableOr[ModelTriggerConfigurationCondition]
    """
    The condition based on which to trigger a job run.
    """

    aliases: VariableOrList[str] = field(default_factory=list)
    """
    Aliases of the model versions to monitor. Can only be used in conjunction with condition MODEL_ALIAS_SET.
    """

    min_time_between_triggers_seconds: VariableOrOptional[int] = None
    """
    If set, the trigger starts a run only after the specified amount of time has passed since
    the last time the trigger fired. The minimum allowed value is 60 seconds.
    """

    securable_name: VariableOrOptional[str] = None
    """
    Name of the securable to monitor ("mycatalog.myschema.mymodel" in the case of model-level triggers,
    "mycatalog.myschema" in the case of schema-level triggers) or empty in the case of metastore-level triggers.
    """

    wait_after_last_change_seconds: VariableOrOptional[int] = None
    """
    If set, the trigger starts a run only after no model updates have occurred for the specified time
    and can be used to wait for a series of model updates before triggering a run. The
    minimum allowed value is 60 seconds.
    """

    @classmethod
    def from_dict(cls, value: "ModelTriggerConfigurationDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "ModelTriggerConfigurationDict":
        return _transform_to_json_value(self)  # type:ignore


class ModelTriggerConfigurationDict(TypedDict, total=False):
    """"""

    condition: VariableOr[ModelTriggerConfigurationConditionParam]
    """
    The condition based on which to trigger a job run.
    """

    aliases: VariableOrList[str]
    """
    Aliases of the model versions to monitor. Can only be used in conjunction with condition MODEL_ALIAS_SET.
    """

    min_time_between_triggers_seconds: VariableOrOptional[int]
    """
    If set, the trigger starts a run only after the specified amount of time has passed since
    the last time the trigger fired. The minimum allowed value is 60 seconds.
    """

    securable_name: VariableOrOptional[str]
    """
    Name of the securable to monitor ("mycatalog.myschema.mymodel" in the case of model-level triggers,
    "mycatalog.myschema" in the case of schema-level triggers) or empty in the case of metastore-level triggers.
    """

    wait_after_last_change_seconds: VariableOrOptional[int]
    """
    If set, the trigger starts a run only after no model updates have occurred for the specified time
    and can be used to wait for a series of model updates before triggering a run. The
    minimum allowed value is 60 seconds.
    """


ModelTriggerConfigurationParam = (
    ModelTriggerConfigurationDict | ModelTriggerConfiguration
)



from enum import Enum
from typing import Literal


class AzureAvailability(Enum):
    """
    Availability type used for all subsequent nodes past the `first_on_demand` ones.
    Note: If `first_on_demand` is zero, this availability type will be used for the entire cluster.
    """

    SPOT_AZURE = "SPOT_AZURE"
    ON_DEMAND_AZURE = "ON_DEMAND_AZURE"
    SPOT_WITH_FALLBACK_AZURE = "SPOT_WITH_FALLBACK_AZURE"


AzureAvailabilityParam = (
    Literal["SPOT_AZURE", "ON_DEMAND_AZURE", "SPOT_WITH_FALLBACK_AZURE"]
    | AzureAvailability
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class DbtPlatformTask:
    """
    :meta private: [EXPERIMENTAL]
    """

    connection_resource_name: VariableOrOptional[str] = None
    """
    The resource name of the UC connection that authenticates the dbt platform for this task
    """

    dbt_platform_job_id: VariableOrOptional[str] = None
    """
    Id of the dbt platform job to be triggered. Specified as a string for maximum compatibility with clients.
    """

    @classmethod
    def from_dict(cls, value: "DbtPlatformTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "DbtPlatformTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class DbtPlatformTaskDict(TypedDict, total=False):
    """"""

    connection_resource_name: VariableOrOptional[str]
    """
    The resource name of the UC connection that authenticates the dbt platform for this task
    """

    dbt_platform_job_id: VariableOrOptional[str]
    """
    Id of the dbt platform job to be triggered. Specified as a string for maximum compatibility with clients.
    """


DbtPlatformTaskParam = DbtPlatformTaskDict | DbtPlatformTask



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValue:
    """
    :meta private: [EXPERIMENTAL]

    [DEPRECATED]
    """

    key: VariableOrOptional[str] = None
    """
    Key for the report parameter, can be a column name or other metadata
    """

    value: VariableOrOptional[str] = None
    """
    Value for the report parameter.
    Possible values it can take are these sql functions:
    1. coalesce(current_offset(), date("YYYY-MM-DD")) -> if current_offset() is null, then the passed date, else current_offset()
    2. current_date()
    3. date_sub(current_date(), x) -> subtract x (some non-negative integer) days from current date
    """

    @classmethod
    def from_dict(
        cls,
        value: "IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueDict",
    ) -> "Self":
        return _transform(cls, value)

    def as_dict(
        self,
    ) -> "IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueDict":
        return _transform_to_json_value(self)  # type:ignore


class IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueDict(
    TypedDict, total=False
):
    """"""

    key: VariableOrOptional[str]
    """
    Key for the report parameter, can be a column name or other metadata
    """

    value: VariableOrOptional[str]
    """
    Value for the report parameter.
    Possible values it can take are these sql functions:
    1. coalesce(current_offset(), date("YYYY-MM-DD")) -> if current_offset() is null, then the passed date, else current_offset()
    2. current_date()
    3. date_sub(current_date(), x) -> subtract x (some non-negative integer) days from current date
    """


IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueParam = (
    IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueDict
    | IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValue
)



from collections.abc import Callable
from dataclasses import dataclass
from typing import TYPE_CHECKING, Generic, Type, TypeVar, overload

from databricks.bundles.core._bundle import Bundle
from databricks.bundles.core._resource import Resource

if TYPE_CHECKING:
    from databricks.bundles.jobs._models.job import Job
    from databricks.bundles.pipelines._models.pipeline import Pipeline
    from databricks.bundles.schemas._models.schema import Schema
    from databricks.bundles.volumes._models.volume import Volume

_T = TypeVar("_T", bound=Resource)


@dataclass(frozen=True)
class ResourceMutator(Generic[_T]):
    """
    Resource mutators are used to modify resources before they are deployed.

    Mutators are applied both to resources defined in YAML and Python.
    Mutators are applied in the order they are defined in databricks.yml.

    Example:

        .. code-block:: yaml

            experimental:
                python:
                    mutators:
                    - "resources:my_job_mutator"

        .. code-block:: python

            from databricks.bundles.core import Bundle, job_mutator
            from databricks.bundles.jobs import Job


            @job_mutator
            def my_job_mutator(bundle: Bundle, job: Job) -> Job:
                return replace(job, name="my_job")

    See :meth:`databricks.bundles.core.job_mutator`.
    """

    resource_type: Type[_T]
    """
    Resource type that this mutator applies to.
    """

    function: Callable
    """
    Underling function that was decorated. Can be accessed for unit-testing.
    """


# Below, we define decorators for each resource type. This approach allows us
# to implement mutators that are only applied for specific resource types.
#
# Alternative approaches considered and rejected during design:
#
# - Inspecting type annotations without decorators.
#   Rationale: Avoid implicit runtime behavior changes based solely on type annotations,
#   especially if a function lacks an explicit decorator.
#
# - Using a universal @mutator decorator.
#   Rationale: Determining whether a mutator is invoked based solely on type annotations
#   was deemed overly implicit and potentially confusing.


@overload
def job_mutator(
    function: Callable[[Bundle, "Job"], "Job"],
) -> ResourceMutator["Job"]: ...


@overload
def job_mutator(function: Callable[["Job"], "Job"]) -> ResourceMutator["Job"]: ...


def job_mutator(function: Callable) -> ResourceMutator["Job"]:
    """
    Decorator for defining a job mutator. Function should return a new instance of the job with the desired changes,
    instead of mutating the input job.

    Example:

    .. code-block:: python

        @job_mutator
        def my_job_mutator(bundle: Bundle, job: Job) -> Job:
            return replace(job, name="my_job")

    :param function: Function that mutates a job.
    """
    from databricks.bundles.jobs._models.job import Job

    return ResourceMutator(resource_type=Job, function=function)


@overload
def pipeline_mutator(
    function: Callable[[Bundle, "Pipeline"], "Pipeline"],
) -> ResourceMutator["Pipeline"]: ...


@overload
def pipeline_mutator(
    function: Callable[["Pipeline"], "Pipeline"],
) -> ResourceMutator["Pipeline"]: ...


def pipeline_mutator(function: Callable) -> ResourceMutator["Pipeline"]:
    """
    Decorator for defining a pipeline mutator. Function should return a new instance of the pipeline with the desired changes,
    instead of mutating the input pipeline.

    Example:

    .. code-block:: python

        @pipeline_mutator
        def my_pipeline_mutator(bundle: Bundle, pipeline: Pipeline) -> Pipeline:
            return replace(pipeline, name="my_job")

    :param function: Function that mutates a pipeline.
    """
    from databricks.bundles.pipelines._models.pipeline import Pipeline

    return ResourceMutator(resource_type=Pipeline, function=function)


@overload
def schema_mutator(
    function: Callable[[Bundle, "Schema"], "Schema"],
) -> ResourceMutator["Schema"]: ...


@overload
def schema_mutator(
    function: Callable[["Schema"], "Schema"],
) -> ResourceMutator["Schema"]: ...


def schema_mutator(function: Callable) -> ResourceMutator["Schema"]:
    """
    Decorator for defining a schema mutator. Function should return a new instance of the schema with the desired changes,
    instead of mutating the input schema.

    Example:

    .. code-block:: python

        @schema_mutator
        def my_schema_mutator(bundle: Bundle, schema: Schema) -> Schema:
            return replace(schema, name="my_schema")

    :param function: Function that mutates a schema.
    """
    from databricks.bundles.schemas._models.schema import Schema

    return ResourceMutator(resource_type=Schema, function=function)


@overload
def volume_mutator(
    function: Callable[[Bundle, "Volume"], "Volume"],
) -> ResourceMutator["Volume"]: ...


@overload
def volume_mutator(
    function: Callable[["Volume"], "Volume"],
) -> ResourceMutator["Volume"]: ...


def volume_mutator(function: Callable) -> ResourceMutator["Volume"]:
    """
    Decorator for defining a volume mutator. Function should return a new instance of the volume with the desired changes,
    instead of mutating the input volume.

    Example:

    .. code-block:: python

        @volume_mutator
        def my_volume_mutator(bundle: Bundle, volume: Volume) -> Volume:
            return replace(volume, name="my_volume")

    :param function: Function that mutates a volume.
    """
    from databricks.bundles.volumes._models.volume import Volume

    return ResourceMutator(resource_type=Volume, function=function)



from enum import Enum
from typing import Literal


class VolumeGrantPrivilege(Enum):
    ALL_PRIVILEGES = "ALL_PRIVILEGES"
    APPLY_TAG = "APPLY_TAG"
    MANAGE = "MANAGE"
    READ_VOLUME = "READ_VOLUME"
    WRITE_VOLUME = "WRITE_VOLUME"


VolumeGrantPrivilegeParam = (
    Literal["ALL_PRIVILEGES", "APPLY_TAG", "MANAGE", "READ_VOLUME", "WRITE_VOLUME"]
    | VolumeGrantPrivilege
)



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._resource import Resource
from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrDict,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.schemas._models.lifecycle import Lifecycle, LifecycleParam
from databricks.bundles.schemas._models.schema_grant import (
    SchemaGrant,
    SchemaGrantParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Schema(Resource):
    """"""

    catalog_name: VariableOr[str]
    """
    Name of parent catalog.
    """

    name: VariableOr[str]
    """
    Name of schema, relative to parent catalog.
    """

    comment: VariableOrOptional[str] = None
    """
    User-provided free-form text description.
    """

    grants: VariableOrList[SchemaGrant] = field(default_factory=list)

    lifecycle: VariableOrOptional[Lifecycle] = None
    """
    Lifecycle is a struct that contains the lifecycle settings for a resource. It controls the behavior of the resource when it is deployed or destroyed.
    """

    properties: VariableOrDict[str] = field(default_factory=dict)

    storage_root: VariableOrOptional[str] = None
    """
    Storage root URL for managed tables within schema.
    """

    @classmethod
    def from_dict(cls, value: "SchemaDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SchemaDict":
        return _transform_to_json_value(self)  # type:ignore


class SchemaDict(TypedDict, total=False):
    """"""

    catalog_name: VariableOr[str]
    """
    Name of parent catalog.
    """

    name: VariableOr[str]
    """
    Name of schema, relative to parent catalog.
    """

    comment: VariableOrOptional[str]
    """
    User-provided free-form text description.
    """

    grants: VariableOrList[SchemaGrantParam]

    lifecycle: VariableOrOptional[LifecycleParam]
    """
    Lifecycle is a struct that contains the lifecycle settings for a resource. It controls the behavior of the resource when it is deployed or destroyed.
    """

    properties: VariableOrDict[str]

    storage_root: VariableOrOptional[str]
    """
    Storage root URL for managed tables within schema.
    """


SchemaParam = SchemaDict | Schema



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.pipelines._models.source_catalog_config import (
    SourceCatalogConfig,
    SourceCatalogConfigParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SourceConfig:
    """"""

    catalog: VariableOrOptional[SourceCatalogConfig] = None
    """
    Catalog-level source configuration parameters
    """

    @classmethod
    def from_dict(cls, value: "SourceConfigDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SourceConfigDict":
        return _transform_to_json_value(self)  # type:ignore


class SourceConfigDict(TypedDict, total=False):
    """"""

    catalog: VariableOrOptional[SourceCatalogConfigParam]
    """
    Catalog-level source configuration parameters
    """


SourceConfigParam = SourceConfigDict | SourceConfig



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr
from databricks.bundles.jobs._models.cluster_spec import ClusterSpec, ClusterSpecParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class JobCluster:
    """"""

    job_cluster_key: VariableOr[str]
    """
    A unique name for the job cluster. This field is required and must be unique within the job.
    `JobTaskSettings` may refer to this field to determine which cluster to launch for the task execution.
    """

    new_cluster: VariableOr[ClusterSpec]
    """
    If new_cluster, a description of a cluster that is created for each task.
    """

    @classmethod
    def from_dict(cls, value: "JobClusterDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "JobClusterDict":
        return _transform_to_json_value(self)  # type:ignore


class JobClusterDict(TypedDict, total=False):
    """"""

    job_cluster_key: VariableOr[str]
    """
    A unique name for the job cluster. This field is required and must be unique within the job.
    `JobTaskSettings` may refer to this field to determine which cluster to launch for the task execution.
    """

    new_cluster: VariableOr[ClusterSpecParam]
    """
    If new_cluster, a description of a cluster that is created for each task.
    """


JobClusterParam = JobClusterDict | JobCluster



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.pipelines._models.dbfs_storage_info import (
    DbfsStorageInfo,
    DbfsStorageInfoParam,
)
from databricks.bundles.pipelines._models.s3_storage_info import (
    S3StorageInfo,
    S3StorageInfoParam,
)
from databricks.bundles.pipelines._models.volumes_storage_info import (
    VolumesStorageInfo,
    VolumesStorageInfoParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class ClusterLogConf:
    """
    Cluster log delivery config
    """

    dbfs: VariableOrOptional[DbfsStorageInfo] = None
    """
    destination needs to be provided. e.g.
    `{ "dbfs" : { "destination" : "dbfs:/home/cluster_log" } }`
    """

    s3: VariableOrOptional[S3StorageInfo] = None
    """
    destination and either the region or endpoint need to be provided. e.g.
    `{ "s3": { "destination" : "s3://cluster_log_bucket/prefix", "region" : "us-west-2" } }`
    Cluster iam role is used to access s3, please make sure the cluster iam role in
    `instance_profile_arn` has permission to write data to the s3 destination.
    """

    volumes: VariableOrOptional[VolumesStorageInfo] = None
    """
    destination needs to be provided, e.g.
    `{ "volumes": { "destination": "/Volumes/catalog/schema/volume/cluster_log" } }`
    """

    @classmethod
    def from_dict(cls, value: "ClusterLogConfDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "ClusterLogConfDict":
        return _transform_to_json_value(self)  # type:ignore


class ClusterLogConfDict(TypedDict, total=False):
    """"""

    dbfs: VariableOrOptional[DbfsStorageInfoParam]
    """
    destination needs to be provided. e.g.
    `{ "dbfs" : { "destination" : "dbfs:/home/cluster_log" } }`
    """

    s3: VariableOrOptional[S3StorageInfoParam]
    """
    destination and either the region or endpoint need to be provided. e.g.
    `{ "s3": { "destination" : "s3://cluster_log_bucket/prefix", "region" : "us-west-2" } }`
    Cluster iam role is used to access s3, please make sure the cluster iam role in
    `instance_profile_arn` has permission to write data to the s3 destination.
    """

    volumes: VariableOrOptional[VolumesStorageInfoParam]
    """
    destination needs to be provided, e.g.
    `{ "volumes": { "destination": "/Volumes/catalog/schema/volume/cluster_log" } }`
    """


ClusterLogConfParam = ClusterLogConfDict | ClusterLogConf



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrDict,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.sql_task_alert import (
    SqlTaskAlert,
    SqlTaskAlertParam,
)
from databricks.bundles.jobs._models.sql_task_dashboard import (
    SqlTaskDashboard,
    SqlTaskDashboardParam,
)
from databricks.bundles.jobs._models.sql_task_file import SqlTaskFile, SqlTaskFileParam
from databricks.bundles.jobs._models.sql_task_query import (
    SqlTaskQuery,
    SqlTaskQueryParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SqlTask:
    """"""

    warehouse_id: VariableOr[str]
    """
    The canonical identifier of the SQL warehouse. Recommended to use with serverless or pro SQL warehouses. Classic SQL warehouses are only supported for SQL alert, dashboard and query tasks and are limited to scheduled single-task jobs.
    """

    alert: VariableOrOptional[SqlTaskAlert] = None
    """
    If alert, indicates that this job must refresh a SQL alert.
    """

    dashboard: VariableOrOptional[SqlTaskDashboard] = None
    """
    If dashboard, indicates that this job must refresh a SQL dashboard.
    """

    file: VariableOrOptional[SqlTaskFile] = None
    """
    If file, indicates that this job runs a SQL file in a remote Git repository.
    """

    parameters: VariableOrDict[str] = field(default_factory=dict)
    """
    Parameters to be used for each run of this job. The SQL alert task does not support custom parameters.
    """

    query: VariableOrOptional[SqlTaskQuery] = None
    """
    If query, indicates that this job must execute a SQL query.
    """

    @classmethod
    def from_dict(cls, value: "SqlTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SqlTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class SqlTaskDict(TypedDict, total=False):
    """"""

    warehouse_id: VariableOr[str]
    """
    The canonical identifier of the SQL warehouse. Recommended to use with serverless or pro SQL warehouses. Classic SQL warehouses are only supported for SQL alert, dashboard and query tasks and are limited to scheduled single-task jobs.
    """

    alert: VariableOrOptional[SqlTaskAlertParam]
    """
    If alert, indicates that this job must refresh a SQL alert.
    """

    dashboard: VariableOrOptional[SqlTaskDashboardParam]
    """
    If dashboard, indicates that this job must refresh a SQL dashboard.
    """

    file: VariableOrOptional[SqlTaskFileParam]
    """
    If file, indicates that this job runs a SQL file in a remote Git repository.
    """

    parameters: VariableOrDict[str]
    """
    Parameters to be used for each run of this job. The SQL alert task does not support custom parameters.
    """

    query: VariableOrOptional[SqlTaskQueryParam]
    """
    If query, indicates that this job must execute a SQL query.
    """


SqlTaskParam = SqlTaskDict | SqlTask



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional
from databricks.bundles.jobs._models.source import Source, SourceParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SqlTaskFile:
    """"""

    path: VariableOr[str]
    """
    Path of the SQL file. Must be relative if the source is a remote Git repository and absolute for workspace paths.
    """

    source: VariableOrOptional[Source] = None
    """
    Optional location type of the SQL file. When set to `WORKSPACE`, the SQL file will be retrieved
    from the local Databricks workspace. When set to `GIT`, the SQL file will be retrieved from a Git repository
    defined in `git_source`. If the value is empty, the task will use `GIT` if `git_source` is defined and `WORKSPACE` otherwise.
    
    * `WORKSPACE`: SQL file is located in Databricks workspace.
    * `GIT`: SQL file is located in cloud Git provider.
    """

    @classmethod
    def from_dict(cls, value: "SqlTaskFileDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SqlTaskFileDict":
        return _transform_to_json_value(self)  # type:ignore


class SqlTaskFileDict(TypedDict, total=False):
    """"""

    path: VariableOr[str]
    """
    Path of the SQL file. Must be relative if the source is a remote Git repository and absolute for workspace paths.
    """

    source: VariableOrOptional[SourceParam]
    """
    Optional location type of the SQL file. When set to `WORKSPACE`, the SQL file will be retrieved
    from the local Databricks workspace. When set to `GIT`, the SQL file will be retrieved from a Git repository
    defined in `git_source`. If the value is empty, the task will use `GIT` if `git_source` is defined and `WORKSPACE` otherwise.
    
    * `WORKSPACE`: SQL file is located in Databricks workspace.
    * `GIT`: SQL file is located in cloud Git provider.
    """


SqlTaskFileParam = SqlTaskFileDict | SqlTaskFile



from enum import Enum
from typing import Literal


class TaskRetryMode(Enum):
    """
    task retry mode of the continuous job
    * NEVER: The failed task will not be retried.
    * ON_FAILURE: Retry a failed task if at least one other task in the job is still running its first attempt.
    When this condition is no longer met or the retry limit is reached, the job run is cancelled and a new run is started.
    """

    NEVER = "NEVER"
    ON_FAILURE = "ON_FAILURE"


TaskRetryModeParam = Literal["NEVER", "ON_FAILURE"] | TaskRetryMode



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional
from databricks.bundles.pipelines._models.table_specific_config import (
    TableSpecificConfig,
    TableSpecificConfigParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SchemaSpec:
    """"""

    destination_catalog: VariableOr[str]
    """
    Required. Destination catalog to store tables.
    """

    destination_schema: VariableOr[str]
    """
    Required. Destination schema to store tables in. Tables with the same name as the source tables are created in this destination schema. The pipeline fails If a table with the same name already exists.
    """

    source_schema: VariableOr[str]
    """
    Required. Schema name in the source database.
    """

    source_catalog: VariableOrOptional[str] = None
    """
    The source catalog name. Might be optional depending on the type of source.
    """

    table_configuration: VariableOrOptional[TableSpecificConfig] = None
    """
    Configuration settings to control the ingestion of tables. These settings are applied to all tables in this schema and override the table_configuration defined in the IngestionPipelineDefinition object.
    """

    @classmethod
    def from_dict(cls, value: "SchemaSpecDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SchemaSpecDict":
        return _transform_to_json_value(self)  # type:ignore


class SchemaSpecDict(TypedDict, total=False):
    """"""

    destination_catalog: VariableOr[str]
    """
    Required. Destination catalog to store tables.
    """

    destination_schema: VariableOr[str]
    """
    Required. Destination schema to store tables in. Tables with the same name as the source tables are created in this destination schema. The pipeline fails If a table with the same name already exists.
    """

    source_schema: VariableOr[str]
    """
    Required. Schema name in the source database.
    """

    source_catalog: VariableOrOptional[str]
    """
    The source catalog name. Might be optional depending on the type of source.
    """

    table_configuration: VariableOrOptional[TableSpecificConfigParam]
    """
    Configuration settings to control the ingestion of tables. These settings are applied to all tables in this schema and override the table_configuration defined in the IngestionPipelineDefinition object.
    """


SchemaSpecParam = SchemaSpecDict | SchemaSpec



from enum import Enum
from typing import Literal


class ModelTriggerConfigurationCondition(Enum):
    """
    :meta private: [EXPERIMENTAL]
    """

    MODEL_CREATED = "MODEL_CREATED"
    MODEL_VERSION_READY = "MODEL_VERSION_READY"
    MODEL_ALIAS_SET = "MODEL_ALIAS_SET"


ModelTriggerConfigurationConditionParam = (
    Literal["MODEL_CREATED", "MODEL_VERSION_READY", "MODEL_ALIAS_SET"]
    | ModelTriggerConfigurationCondition
)



from enum import Enum
from typing import Literal


class GcpAvailability(Enum):
    """
    This field determines whether the instance pool will contain preemptible
    VMs, on-demand VMs, or preemptible VMs with a fallback to on-demand VMs if the former is unavailable.
    """

    PREEMPTIBLE_GCP = "PREEMPTIBLE_GCP"
    ON_DEMAND_GCP = "ON_DEMAND_GCP"
    PREEMPTIBLE_WITH_FALLBACK_GCP = "PREEMPTIBLE_WITH_FALLBACK_GCP"


GcpAvailabilityParam = (
    Literal["PREEMPTIBLE_GCP", "ON_DEMAND_GCP", "PREEMPTIBLE_WITH_FALLBACK_GCP"]
    | GcpAvailability
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class GcsStorageInfo:
    """
    A storage location in Google Cloud Platform's GCS
    """

    destination: VariableOr[str]
    """
    GCS destination/URI, e.g. `gs://my-bucket/some-prefix`
    """

    @classmethod
    def from_dict(cls, value: "GcsStorageInfoDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "GcsStorageInfoDict":
        return _transform_to_json_value(self)  # type:ignore


class GcsStorageInfoDict(TypedDict, total=False):
    """"""

    destination: VariableOr[str]
    """
    GCS destination/URI, e.g. `gs://my-bucket/some-prefix`
    """


GcsStorageInfoParam = GcsStorageInfoDict | GcsStorageInfo



__all__ = [
    "Adlsgen2Info",
    "Adlsgen2InfoDict",
    "Adlsgen2InfoParam",
    "AwsAttributes",
    "AwsAttributesDict",
    "AwsAttributesParam",
    "AwsAvailability",
    "AwsAvailabilityParam",
    "AzureAttributes",
    "AzureAttributesDict",
    "AzureAttributesParam",
    "AzureAvailability",
    "AzureAvailabilityParam",
    "ClusterLogConf",
    "ClusterLogConfDict",
    "ClusterLogConfParam",
    "ConnectionParameters",
    "ConnectionParametersDict",
    "ConnectionParametersParam",
    "DayOfWeek",
    "DayOfWeekParam",
    "DbfsStorageInfo",
    "DbfsStorageInfoDict",
    "DbfsStorageInfoParam",
    "EbsVolumeType",
    "EbsVolumeTypeParam",
    "EventLogSpec",
    "EventLogSpecDict",
    "EventLogSpecParam",
    "FileLibrary",
    "FileLibraryDict",
    "FileLibraryParam",
    "Filters",
    "FiltersDict",
    "FiltersParam",
    "GcpAttributes",
    "GcpAttributesDict",
    "GcpAttributesParam",
    "GcpAvailability",
    "GcpAvailabilityParam",
    "GcsStorageInfo",
    "GcsStorageInfoDict",
    "GcsStorageInfoParam",
    "IngestionConfig",
    "IngestionConfigDict",
    "IngestionConfigParam",
    "IngestionGatewayPipelineDefinition",
    "IngestionGatewayPipelineDefinitionDict",
    "IngestionGatewayPipelineDefinitionParam",
    "IngestionPipelineDefinition",
    "IngestionPipelineDefinitionDict",
    "IngestionPipelineDefinitionParam",
    "IngestionPipelineDefinitionTableSpecificConfigQueryBasedConnectorConfig",
    "IngestionPipelineDefinitionTableSpecificConfigQueryBasedConnectorConfigDict",
    "IngestionPipelineDefinitionTableSpecificConfigQueryBasedConnectorConfigParam",
    "IngestionPipelineDefinitionWorkdayReportParameters",
    "IngestionPipelineDefinitionWorkdayReportParametersDict",
    "IngestionPipelineDefinitionWorkdayReportParametersParam",
    "IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValue",
    "IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueDict",
    "IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueParam",
    "InitScriptInfo",
    "InitScriptInfoDict",
    "InitScriptInfoParam",
    "Lifecycle",
    "LifecycleDict",
    "LifecycleParam",
    "LocalFileInfo",
    "LocalFileInfoDict",
    "LocalFileInfoParam",
    "LogAnalyticsInfo",
    "LogAnalyticsInfoDict",
    "LogAnalyticsInfoParam",
    "MavenLibrary",
    "MavenLibraryDict",
    "MavenLibraryParam",
    "NotebookLibrary",
    "NotebookLibraryDict",
    "NotebookLibraryParam",
    "Notifications",
    "NotificationsDict",
    "NotificationsParam",
    "PathPattern",
    "PathPatternDict",
    "PathPatternParam",
    "Pipeline",
    "PipelineCluster",
    "PipelineClusterAutoscale",
    "PipelineClusterAutoscaleDict",
    "PipelineClusterAutoscaleMode",
    "PipelineClusterAutoscaleModeParam",
    "PipelineClusterAutoscaleParam",
    "PipelineClusterDict",
    "PipelineClusterParam",
    "PipelineDict",
    "PipelineLibrary",
    "PipelineLibraryDict",
    "PipelineLibraryParam",
    "PipelineParam",
    "PipelinePermission",
    "PipelinePermissionDict",
    "PipelinePermissionLevel",
    "PipelinePermissionLevelParam",
    "PipelinePermissionParam",
    "PipelinesEnvironment",
    "PipelinesEnvironmentDict",
    "PipelinesEnvironmentParam",
    "PostgresCatalogConfig",
    "PostgresCatalogConfigDict",
    "PostgresCatalogConfigParam",
    "PostgresSlotConfig",
    "PostgresSlotConfigDict",
    "PostgresSlotConfigParam",
    "ReportSpec",
    "ReportSpecDict",
    "ReportSpecParam",
    "RestartWindow",
    "RestartWindowDict",
    "RestartWindowParam",
    "RunAs",
    "RunAsDict",
    "RunAsParam",
    "S3StorageInfo",
    "S3StorageInfoDict",
    "S3StorageInfoParam",
    "SchemaSpec",
    "SchemaSpecDict",
    "SchemaSpecParam",
    "SourceCatalogConfig",
    "SourceCatalogConfigDict",
    "SourceCatalogConfigParam",
    "SourceConfig",
    "SourceConfigDict",
    "SourceConfigParam",
    "TableSpec",
    "TableSpecDict",
    "TableSpecParam",
    "TableSpecificConfig",
    "TableSpecificConfigDict",
    "TableSpecificConfigParam",
    "TableSpecificConfigScdType",
    "TableSpecificConfigScdTypeParam",
    "VolumesStorageInfo",
    "VolumesStorageInfoDict",
    "VolumesStorageInfoParam",
    "WorkspaceStorageInfo",
    "WorkspaceStorageInfoDict",
    "WorkspaceStorageInfoParam",
]


from databricks.bundles.pipelines._models.adlsgen2_info import (
    Adlsgen2Info,
    Adlsgen2InfoDict,
    Adlsgen2InfoParam,
)
from databricks.bundles.pipelines._models.aws_attributes import (
    AwsAttributes,
    AwsAttributesDict,
    AwsAttributesParam,
)
from databricks.bundles.pipelines._models.aws_availability import (
    AwsAvailability,
    AwsAvailabilityParam,
)
from databricks.bundles.pipelines._models.azure_attributes import (
    AzureAttributes,
    AzureAttributesDict,
    AzureAttributesParam,
)
from databricks.bundles.pipelines._models.azure_availability import (
    AzureAvailability,
    AzureAvailabilityParam,
)
from databricks.bundles.pipelines._models.cluster_log_conf import (
    ClusterLogConf,
    ClusterLogConfDict,
    ClusterLogConfParam,
)
from databricks.bundles.pipelines._models.connection_parameters import (
    ConnectionParameters,
    ConnectionParametersDict,
    ConnectionParametersParam,
)
from databricks.bundles.pipelines._models.day_of_week import DayOfWeek, DayOfWeekParam
from databricks.bundles.pipelines._models.dbfs_storage_info import (
    DbfsStorageInfo,
    DbfsStorageInfoDict,
    DbfsStorageInfoParam,
)
from databricks.bundles.pipelines._models.ebs_volume_type import (
    EbsVolumeType,
    EbsVolumeTypeParam,
)
from databricks.bundles.pipelines._models.event_log_spec import (
    EventLogSpec,
    EventLogSpecDict,
    EventLogSpecParam,
)
from databricks.bundles.pipelines._models.file_library import (
    FileLibrary,
    FileLibraryDict,
    FileLibraryParam,
)
from databricks.bundles.pipelines._models.filters import (
    Filters,
    FiltersDict,
    FiltersParam,
)
from databricks.bundles.pipelines._models.gcp_attributes import (
    GcpAttributes,
    GcpAttributesDict,
    GcpAttributesParam,
)
from databricks.bundles.pipelines._models.gcp_availability import (
    GcpAvailability,
    GcpAvailabilityParam,
)
from databricks.bundles.pipelines._models.gcs_storage_info import (
    GcsStorageInfo,
    GcsStorageInfoDict,
    GcsStorageInfoParam,
)
from databricks.bundles.pipelines._models.ingestion_config import (
    IngestionConfig,
    IngestionConfigDict,
    IngestionConfigParam,
)
from databricks.bundles.pipelines._models.ingestion_gateway_pipeline_definition import (
    IngestionGatewayPipelineDefinition,
    IngestionGatewayPipelineDefinitionDict,
    IngestionGatewayPipelineDefinitionParam,
)
from databricks.bundles.pipelines._models.ingestion_pipeline_definition import (
    IngestionPipelineDefinition,
    IngestionPipelineDefinitionDict,
    IngestionPipelineDefinitionParam,
)
from databricks.bundles.pipelines._models.ingestion_pipeline_definition_table_specific_config_query_based_connector_config import (
    IngestionPipelineDefinitionTableSpecificConfigQueryBasedConnectorConfig,
    IngestionPipelineDefinitionTableSpecificConfigQueryBasedConnectorConfigDict,
    IngestionPipelineDefinitionTableSpecificConfigQueryBasedConnectorConfigParam,
)
from databricks.bundles.pipelines._models.ingestion_pipeline_definition_workday_report_parameters import (
    IngestionPipelineDefinitionWorkdayReportParameters,
    IngestionPipelineDefinitionWorkdayReportParametersDict,
    IngestionPipelineDefinitionWorkdayReportParametersParam,
)
from databricks.bundles.pipelines._models.ingestion_pipeline_definition_workday_report_parameters_query_key_value import (
    IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValue,
    IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueDict,
    IngestionPipelineDefinitionWorkdayReportParametersQueryKeyValueParam,
)
from databricks.bundles.pipelines._models.init_script_info import (
    InitScriptInfo,
    InitScriptInfoDict,
    InitScriptInfoParam,
)
from databricks.bundles.pipelines._models.lifecycle import (
    Lifecycle,
    LifecycleDict,
    LifecycleParam,
)
from databricks.bundles.pipelines._models.local_file_info import (
    LocalFileInfo,
    LocalFileInfoDict,
    LocalFileInfoParam,
)
from databricks.bundles.pipelines._models.log_analytics_info import (
    LogAnalyticsInfo,
    LogAnalyticsInfoDict,
    LogAnalyticsInfoParam,
)
from databricks.bundles.pipelines._models.maven_library import (
    MavenLibrary,
    MavenLibraryDict,
    MavenLibraryParam,
)
from databricks.bundles.pipelines._models.notebook_library import (
    NotebookLibrary,
    NotebookLibraryDict,
    NotebookLibraryParam,
)
from databricks.bundles.pipelines._models.notifications import (
    Notifications,
    NotificationsDict,
    NotificationsParam,
)
from databricks.bundles.pipelines._models.path_pattern import (
    PathPattern,
    PathPatternDict,
    PathPatternParam,
)
from databricks.bundles.pipelines._models.pipeline import (
    Pipeline,
    PipelineDict,
    PipelineParam,
)
from databricks.bundles.pipelines._models.pipeline_cluster import (
    PipelineCluster,
    PipelineClusterDict,
    PipelineClusterParam,
)
from databricks.bundles.pipelines._models.pipeline_cluster_autoscale import (
    PipelineClusterAutoscale,
    PipelineClusterAutoscaleDict,
    PipelineClusterAutoscaleParam,
)
from databricks.bundles.pipelines._models.pipeline_cluster_autoscale_mode import (
    PipelineClusterAutoscaleMode,
    PipelineClusterAutoscaleModeParam,
)
from databricks.bundles.pipelines._models.pipeline_library import (
    PipelineLibrary,
    PipelineLibraryDict,
    PipelineLibraryParam,
)
from databricks.bundles.pipelines._models.pipeline_permission import (
    PipelinePermission,
    PipelinePermissionDict,
    PipelinePermissionParam,
)
from databricks.bundles.pipelines._models.pipeline_permission_level import (
    PipelinePermissionLevel,
    PipelinePermissionLevelParam,
)
from databricks.bundles.pipelines._models.pipelines_environment import (
    PipelinesEnvironment,
    PipelinesEnvironmentDict,
    PipelinesEnvironmentParam,
)
from databricks.bundles.pipelines._models.postgres_catalog_config import (
    PostgresCatalogConfig,
    PostgresCatalogConfigDict,
    PostgresCatalogConfigParam,
)
from databricks.bundles.pipelines._models.postgres_slot_config import (
    PostgresSlotConfig,
    PostgresSlotConfigDict,
    PostgresSlotConfigParam,
)
from databricks.bundles.pipelines._models.report_spec import (
    ReportSpec,
    ReportSpecDict,
    ReportSpecParam,
)
from databricks.bundles.pipelines._models.restart_window import (
    RestartWindow,
    RestartWindowDict,
    RestartWindowParam,
)
from databricks.bundles.pipelines._models.run_as import RunAs, RunAsDict, RunAsParam
from databricks.bundles.pipelines._models.s3_storage_info import (
    S3StorageInfo,
    S3StorageInfoDict,
    S3StorageInfoParam,
)
from databricks.bundles.pipelines._models.schema_spec import (
    SchemaSpec,
    SchemaSpecDict,
    SchemaSpecParam,
)
from databricks.bundles.pipelines._models.source_catalog_config import (
    SourceCatalogConfig,
    SourceCatalogConfigDict,
    SourceCatalogConfigParam,
)
from databricks.bundles.pipelines._models.source_config import (
    SourceConfig,
    SourceConfigDict,
    SourceConfigParam,
)
from databricks.bundles.pipelines._models.table_spec import (
    TableSpec,
    TableSpecDict,
    TableSpecParam,
)
from databricks.bundles.pipelines._models.table_specific_config import (
    TableSpecificConfig,
    TableSpecificConfigDict,
    TableSpecificConfigParam,
)
from databricks.bundles.pipelines._models.table_specific_config_scd_type import (
    TableSpecificConfigScdType,
    TableSpecificConfigScdTypeParam,
)
from databricks.bundles.pipelines._models.volumes_storage_info import (
    VolumesStorageInfo,
    VolumesStorageInfoDict,
    VolumesStorageInfoParam,
)
from databricks.bundles.pipelines._models.workspace_storage_info import (
    WorkspaceStorageInfo,
    WorkspaceStorageInfoDict,
    WorkspaceStorageInfoParam,
)



from enum import Enum
from typing import Literal


class Condition(Enum):
    ANY_UPDATED = "ANY_UPDATED"
    ALL_UPDATED = "ALL_UPDATED"


ConditionParam = Literal["ANY_UPDATED", "ALL_UPDATED"] | Condition



from enum import Enum
from typing import Literal


class SchemaGrantPrivilege(Enum):
    ALL_PRIVILEGES = "ALL_PRIVILEGES"
    APPLY_TAG = "APPLY_TAG"
    CREATE_FUNCTION = "CREATE_FUNCTION"
    CREATE_TABLE = "CREATE_TABLE"
    CREATE_VOLUME = "CREATE_VOLUME"
    MANAGE = "MANAGE"
    USE_SCHEMA = "USE_SCHEMA"
    EXECUTE = "EXECUTE"
    MODIFY = "MODIFY"
    REFRESH = "REFRESH"
    SELECT = "SELECT"
    READ_VOLUME = "READ_VOLUME"
    WRITE_VOLUME = "WRITE_VOLUME"


SchemaGrantPrivilegeParam = (
    Literal[
        "ALL_PRIVILEGES",
        "APPLY_TAG",
        "CREATE_FUNCTION",
        "CREATE_TABLE",
        "CREATE_VOLUME",
        "MANAGE",
        "USE_SCHEMA",
        "EXECUTE",
        "MODIFY",
        "REFRESH",
        "SELECT",
        "READ_VOLUME",
        "WRITE_VOLUME",
    ]
    | SchemaGrantPrivilege
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.jobs._models.subscription import (
    Subscription,
    SubscriptionParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class DashboardTask:
    """
    Configures the Lakeview Dashboard job task type.
    """

    dashboard_id: VariableOrOptional[str] = None

    subscription: VariableOrOptional[Subscription] = None

    warehouse_id: VariableOrOptional[str] = None
    """
    Optional: The warehouse id to execute the dashboard with for the schedule.
    If not specified, the default warehouse of the dashboard will be used.
    """

    @classmethod
    def from_dict(cls, value: "DashboardTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "DashboardTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class DashboardTaskDict(TypedDict, total=False):
    """"""

    dashboard_id: VariableOrOptional[str]

    subscription: VariableOrOptional[SubscriptionParam]

    warehouse_id: VariableOrOptional[str]
    """
    Optional: The warehouse id to execute the dashboard with for the schedule.
    If not specified, the default warehouse of the dashboard will be used.
    """


DashboardTaskParam = DashboardTaskDict | DashboardTask



from enum import Enum
from typing import Literal


class JobsHealthOperator(Enum):
    """
    Specifies the operator used to compare the health metric value with the specified threshold.
    """

    GREATER_THAN = "GREATER_THAN"


JobsHealthOperatorParam = Literal["GREATER_THAN"] | JobsHealthOperator



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class EventLogSpec:
    """
    Configurable event log parameters.
    """

    catalog: VariableOrOptional[str] = None
    """
    The UC catalog the event log is published under.
    """

    name: VariableOrOptional[str] = None
    """
    The name the event log is published to in UC.
    """

    schema: VariableOrOptional[str] = None
    """
    The UC schema the event log is published under.
    """

    @classmethod
    def from_dict(cls, value: "EventLogSpecDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "EventLogSpecDict":
        return _transform_to_json_value(self)  # type:ignore


class EventLogSpecDict(TypedDict, total=False):
    """"""

    catalog: VariableOrOptional[str]
    """
    The UC catalog the event log is published under.
    """

    name: VariableOrOptional[str]
    """
    The name the event log is published to in UC.
    """

    schema: VariableOrOptional[str]
    """
    The UC schema the event log is published under.
    """


EventLogSpecParam = EventLogSpecDict | EventLogSpec



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList, VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Environment:
    """
    The environment entity used to preserve serverless environment side panel, jobs' environment for non-notebook task, and DLT's environment for classic and serverless pipelines.
    In this minimal environment spec, only pip dependencies are supported.
    """

    client: VariableOrOptional[str] = None
    """
    [DEPRECATED] Use `environment_version` instead.
    """

    dependencies: VariableOrList[str] = field(default_factory=list)
    """
    List of pip dependencies, as supported by the version of pip in this environment.
    """

    environment_version: VariableOrOptional[str] = None
    """
    Required. Environment version used by the environment.
    Each version comes with a specific Python version and a set of Python packages.
    The version is a string, consisting of an integer.
    """

    java_dependencies: VariableOrList[str] = field(default_factory=list)

    @classmethod
    def from_dict(cls, value: "EnvironmentDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "EnvironmentDict":
        return _transform_to_json_value(self)  # type:ignore


class EnvironmentDict(TypedDict, total=False):
    """"""

    client: VariableOrOptional[str]
    """
    [DEPRECATED] Use `environment_version` instead.
    """

    dependencies: VariableOrList[str]
    """
    List of pip dependencies, as supported by the version of pip in this environment.
    """

    environment_version: VariableOrOptional[str]
    """
    Required. Environment version used by the environment.
    Each version comes with a specific Python version and a set of Python packages.
    The version is a string, consisting of an integer.
    """

    java_dependencies: VariableOrList[str]


EnvironmentParam = EnvironmentDict | Environment



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.jobs._models.pause_status import PauseStatus, PauseStatusParam
from databricks.bundles.jobs._models.task_retry_mode import (
    TaskRetryMode,
    TaskRetryModeParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Continuous:
    """"""

    pause_status: VariableOrOptional[PauseStatus] = None
    """
    Indicate whether the continuous execution of the job is paused or not. Defaults to UNPAUSED.
    """

    task_retry_mode: VariableOrOptional[TaskRetryMode] = None
    """
    Indicate whether the continuous job is applying task level retries or not. Defaults to NEVER.
    """

    @classmethod
    def from_dict(cls, value: "ContinuousDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "ContinuousDict":
        return _transform_to_json_value(self)  # type:ignore


class ContinuousDict(TypedDict, total=False):
    """"""

    pause_status: VariableOrOptional[PauseStatusParam]
    """
    Indicate whether the continuous execution of the job is paused or not. Defaults to UNPAUSED.
    """

    task_retry_mode: VariableOrOptional[TaskRetryModeParam]
    """
    Indicate whether the continuous job is applying task level retries or not. Defaults to NEVER.
    """


ContinuousParam = ContinuousDict | Continuous



from enum import Enum
from typing import Literal


class ConditionTaskOp(Enum):
    """
    * `EQUAL_TO`, `NOT_EQUAL` operators perform string comparison of their operands. This means that `“12.0” == “12”` will evaluate to `false`.
    * `GREATER_THAN`, `GREATER_THAN_OR_EQUAL`, `LESS_THAN`, `LESS_THAN_OR_EQUAL` operators perform numeric comparison of their operands. `“12.0” >= “12”` will evaluate to `true`, `“10.0” >= “12”` will evaluate to `false`.

    The boolean comparison to task values can be implemented with operators `EQUAL_TO`, `NOT_EQUAL`. If a task value was set to a boolean value, it will be serialized to `“true”` or `“false”` for the comparison.
    """

    EQUAL_TO = "EQUAL_TO"
    GREATER_THAN = "GREATER_THAN"
    GREATER_THAN_OR_EQUAL = "GREATER_THAN_OR_EQUAL"
    LESS_THAN = "LESS_THAN"
    LESS_THAN_OR_EQUAL = "LESS_THAN_OR_EQUAL"
    NOT_EQUAL = "NOT_EQUAL"


ConditionTaskOpParam = (
    Literal[
        "EQUAL_TO",
        "GREATER_THAN",
        "GREATER_THAN_OR_EQUAL",
        "LESS_THAN",
        "LESS_THAN_OR_EQUAL",
        "NOT_EQUAL",
    ]
    | ConditionTaskOp
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class QueueSettings:
    """"""

    enabled: VariableOr[bool]
    """
    If true, enable queueing for the job. This is a required field.
    """

    @classmethod
    def from_dict(cls, value: "QueueSettingsDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "QueueSettingsDict":
        return _transform_to_json_value(self)  # type:ignore


class QueueSettingsDict(TypedDict, total=False):
    """"""

    enabled: VariableOr[bool]
    """
    If true, enable queueing for the job. This is a required field.
    """


QueueSettingsParam = QueueSettingsDict | QueueSettings



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class TaskNotificationSettings:
    """"""

    alert_on_last_attempt: VariableOrOptional[bool] = None
    """
    If true, do not send notifications to recipients specified in `on_start` for the retried runs and do not send notifications to recipients specified in `on_failure` until the last retry of the run.
    """

    no_alert_for_canceled_runs: VariableOrOptional[bool] = None
    """
    If true, do not send notifications to recipients specified in `on_failure` if the run is canceled.
    """

    no_alert_for_skipped_runs: VariableOrOptional[bool] = None
    """
    If true, do not send notifications to recipients specified in `on_failure` if the run is skipped.
    """

    @classmethod
    def from_dict(cls, value: "TaskNotificationSettingsDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "TaskNotificationSettingsDict":
        return _transform_to_json_value(self)  # type:ignore


class TaskNotificationSettingsDict(TypedDict, total=False):
    """"""

    alert_on_last_attempt: VariableOrOptional[bool]
    """
    If true, do not send notifications to recipients specified in `on_start` for the retried runs and do not send notifications to recipients specified in `on_failure` until the last retry of the run.
    """

    no_alert_for_canceled_runs: VariableOrOptional[bool]
    """
    If true, do not send notifications to recipients specified in `on_failure` if the run is canceled.
    """

    no_alert_for_skipped_runs: VariableOrOptional[bool]
    """
    If true, do not send notifications to recipients specified in `on_failure` if the run is skipped.
    """


TaskNotificationSettingsParam = TaskNotificationSettingsDict | TaskNotificationSettings



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrDict,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.source import Source, SourceParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class NotebookTask:
    """"""

    notebook_path: VariableOr[str]
    """
    The path of the notebook to be run in the Databricks workspace or remote repository.
    For notebooks stored in the Databricks workspace, the path must be absolute and begin with a slash.
    For notebooks stored in a remote repository, the path must be relative. This field is required.
    """

    base_parameters: VariableOrDict[str] = field(default_factory=dict)
    """
    Base parameters to be used for each run of this job. If the run is initiated by a call to :method:jobs/run
    Now with parameters specified, the two parameters maps are merged. If the same key is specified in
    `base_parameters` and in `run-now`, the value from `run-now` is used.
    Use [Task parameter variables](https://docs.databricks.com/jobs.html#parameter-variables) to set parameters containing information about job runs.
    
    If the notebook takes a parameter that is not specified in the job’s `base_parameters` or the `run-now` override parameters,
    the default value from the notebook is used.
    
    Retrieve these parameters in a notebook using [dbutils.widgets.get](https://docs.databricks.com/dev-tools/databricks-utils.html#dbutils-widgets).
    
    The JSON representation of this field cannot exceed 1MB.
    """

    source: VariableOrOptional[Source] = None
    """
    Optional location type of the notebook. When set to `WORKSPACE`, the notebook will be retrieved from the local Databricks workspace. When set to `GIT`, the notebook will be retrieved from a Git repository
    defined in `git_source`. If the value is empty, the task will use `GIT` if `git_source` is defined and `WORKSPACE` otherwise.
    * `WORKSPACE`: Notebook is located in Databricks workspace.
    * `GIT`: Notebook is located in cloud Git provider.
    """

    warehouse_id: VariableOrOptional[str] = None
    """
    Optional `warehouse_id` to run the notebook on a SQL warehouse. Classic SQL warehouses are NOT supported, please use serverless or pro SQL warehouses.
    
    Note that SQL warehouses only support SQL cells; if the notebook contains non-SQL cells, the run will fail.
    """

    @classmethod
    def from_dict(cls, value: "NotebookTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "NotebookTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class NotebookTaskDict(TypedDict, total=False):
    """"""

    notebook_path: VariableOr[str]
    """
    The path of the notebook to be run in the Databricks workspace or remote repository.
    For notebooks stored in the Databricks workspace, the path must be absolute and begin with a slash.
    For notebooks stored in a remote repository, the path must be relative. This field is required.
    """

    base_parameters: VariableOrDict[str]
    """
    Base parameters to be used for each run of this job. If the run is initiated by a call to :method:jobs/run
    Now with parameters specified, the two parameters maps are merged. If the same key is specified in
    `base_parameters` and in `run-now`, the value from `run-now` is used.
    Use [Task parameter variables](https://docs.databricks.com/jobs.html#parameter-variables) to set parameters containing information about job runs.
    
    If the notebook takes a parameter that is not specified in the job’s `base_parameters` or the `run-now` override parameters,
    the default value from the notebook is used.
    
    Retrieve these parameters in a notebook using [dbutils.widgets.get](https://docs.databricks.com/dev-tools/databricks-utils.html#dbutils-widgets).
    
    The JSON representation of this field cannot exceed 1MB.
    """

    source: VariableOrOptional[SourceParam]
    """
    Optional location type of the notebook. When set to `WORKSPACE`, the notebook will be retrieved from the local Databricks workspace. When set to `GIT`, the notebook will be retrieved from a Git repository
    defined in `git_source`. If the value is empty, the task will use `GIT` if `git_source` is defined and `WORKSPACE` otherwise.
    * `WORKSPACE`: Notebook is located in Databricks workspace.
    * `GIT`: Notebook is located in cloud Git provider.
    """

    warehouse_id: VariableOrOptional[str]
    """
    Optional `warehouse_id` to run the notebook on a SQL warehouse. Classic SQL warehouses are NOT supported, please use serverless or pro SQL warehouses.
    
    Note that SQL warehouses only support SQL cells; if the notebook contains non-SQL cells, the run will fail.
    """


NotebookTaskParam = NotebookTaskDict | NotebookTask



from enum import Enum
from typing import Literal


class PipelineClusterAutoscaleMode(Enum):
    """
    Databricks Enhanced Autoscaling optimizes cluster utilization by automatically
    allocating cluster resources based on workload volume, with minimal impact to
    the data processing latency of your pipelines. Enhanced Autoscaling is available
    for `updates` clusters only. The legacy autoscaling feature is used for `maintenance`
    clusters.
    """

    ENHANCED = "ENHANCED"
    LEGACY = "LEGACY"


PipelineClusterAutoscaleModeParam = (
    Literal["ENHANCED", "LEGACY"] | PipelineClusterAutoscaleMode
)



import inspect
import os
from dataclasses import dataclass
from typing import Callable, Optional

__all__ = [
    "Location",
]


@dataclass(kw_only=True, frozen=True)
class Location:
    file: str

    line: Optional[int] = None
    """
    Line number in the file. Line numbers are 1-based and should be greater than 0.
    """

    column: Optional[int] = None
    """
    Column number in the line. Column numbers are 1-based and should be greater than 0.
    """

    def __post_init__(self):
        if self.line is not None and self.line < 1:
            raise ValueError(f"Line number must be greater than 0, got {self.line}")

        if self.column is not None and self.column < 1:
            raise ValueError(f"Column number must be greater than 0, got {self.column}")

    @staticmethod
    def from_callable(fn: Callable) -> Optional["Location"]:
        """
        Capture location of callable. This is useful for creating
        diagnostics of decorated functions.
        """

        code = hasattr(fn, "__code__") and getattr(fn, "__code__")

        if not code:
            return None

        file = code.co_filename
        if file and file.startswith(os.getcwd()):
            # simplify path if we can
            file = os.path.relpath(file, os.getcwd())

        return Location(
            file=file,
            line=code.co_firstlineno,
            column=1,  # there is no way to get column
        )

    @staticmethod
    def from_stack_frame(depth: int = 0) -> "Location":
        """
        Capture location of the caller
        """

        stack = inspect.stack()
        frame = stack[1 + depth].frame

        return Location(
            file=frame.f_code.co_filename,
            line=frame.f_lineno,
            column=1,  # there is no way to get column
        )

    def as_dict(self) -> dict:
        def omit_none(values: dict):
            return {key: value for key, value in values.items() if value is not None}

        return omit_none(
            {
                "file": self.file,
                "line": self.line,
                "column": self.column,
            }
        )



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PostgresSlotConfig:
    """
    PostgresSlotConfig contains the configuration for a Postgres logical replication slot
    """

    publication_name: VariableOrOptional[str] = None
    """
    The name of the publication to use for the Postgres source
    """

    slot_name: VariableOrOptional[str] = None
    """
    The name of the logical replication slot to use for the Postgres source
    """

    @classmethod
    def from_dict(cls, value: "PostgresSlotConfigDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PostgresSlotConfigDict":
        return _transform_to_json_value(self)  # type:ignore


class PostgresSlotConfigDict(TypedDict, total=False):
    """"""

    publication_name: VariableOrOptional[str]
    """
    The name of the publication to use for the Postgres source
    """

    slot_name: VariableOrOptional[str]
    """
    The name of the logical replication slot to use for the Postgres source
    """


PostgresSlotConfigParam = PostgresSlotConfigDict | PostgresSlotConfig



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList, VariableOrOptional
from databricks.bundles.jobs._models.source import Source, SourceParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class DbtTask:
    """"""

    catalog: VariableOrOptional[str] = None
    """
    Optional name of the catalog to use. The value is the top level in the 3-level namespace of Unity Catalog (catalog / schema / relation). The catalog value can only be specified if a warehouse_id is specified. Requires dbt-databricks >= 1.1.1.
    """

    commands: VariableOrList[str] = field(default_factory=list)
    """
    A list of dbt commands to execute. All commands must start with `dbt`. This parameter must not be empty. A maximum of up to 10 commands can be provided.
    """

    profiles_directory: VariableOrOptional[str] = None
    """
    Optional (relative) path to the profiles directory. Can only be specified if no warehouse_id is specified. If no warehouse_id is specified and this folder is unset, the root directory is used.
    """

    project_directory: VariableOrOptional[str] = None
    """
    Path to the project directory. Optional for Git sourced tasks, in which
    case if no value is provided, the root of the Git repository is used.
    """

    schema: VariableOrOptional[str] = None
    """
    Optional schema to write to. This parameter is only used when a warehouse_id is also provided. If not provided, the `default` schema is used.
    """

    source: VariableOrOptional[Source] = None
    """
    Optional location type of the project directory. When set to `WORKSPACE`, the project will be retrieved
    from the local Databricks workspace. When set to `GIT`, the project will be retrieved from a Git repository
    defined in `git_source`. If the value is empty, the task will use `GIT` if `git_source` is defined and `WORKSPACE` otherwise.
    
    * `WORKSPACE`: Project is located in Databricks workspace.
    * `GIT`: Project is located in cloud Git provider.
    """

    warehouse_id: VariableOrOptional[str] = None
    """
    ID of the SQL warehouse to connect to. If provided, we automatically generate and provide the profile and connection details to dbt. It can be overridden on a per-command basis by using the `--profiles-dir` command line argument.
    """

    @classmethod
    def from_dict(cls, value: "DbtTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "DbtTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class DbtTaskDict(TypedDict, total=False):
    """"""

    catalog: VariableOrOptional[str]
    """
    Optional name of the catalog to use. The value is the top level in the 3-level namespace of Unity Catalog (catalog / schema / relation). The catalog value can only be specified if a warehouse_id is specified. Requires dbt-databricks >= 1.1.1.
    """

    commands: VariableOrList[str]
    """
    A list of dbt commands to execute. All commands must start with `dbt`. This parameter must not be empty. A maximum of up to 10 commands can be provided.
    """

    profiles_directory: VariableOrOptional[str]
    """
    Optional (relative) path to the profiles directory. Can only be specified if no warehouse_id is specified. If no warehouse_id is specified and this folder is unset, the root directory is used.
    """

    project_directory: VariableOrOptional[str]
    """
    Path to the project directory. Optional for Git sourced tasks, in which
    case if no value is provided, the root of the Git repository is used.
    """

    schema: VariableOrOptional[str]
    """
    Optional schema to write to. This parameter is only used when a warehouse_id is also provided. If not provided, the `default` schema is used.
    """

    source: VariableOrOptional[SourceParam]
    """
    Optional location type of the project directory. When set to `WORKSPACE`, the project will be retrieved
    from the local Databricks workspace. When set to `GIT`, the project will be retrieved from a Git repository
    defined in `git_source`. If the value is empty, the task will use `GIT` if `git_source` is defined and `WORKSPACE` otherwise.
    
    * `WORKSPACE`: Project is located in Databricks workspace.
    * `GIT`: Project is located in cloud Git provider.
    """

    warehouse_id: VariableOrOptional[str]
    """
    ID of the SQL warehouse to connect to. If provided, we automatically generate and provide the profile and connection details to dbt. It can be overridden on a per-command basis by using the `--profiles-dir` command line argument.
    """


DbtTaskParam = DbtTaskDict | DbtTask



from enum import Enum
from typing import Literal


class JobsHealthMetric(Enum):
    """
    Specifies the health metric that is being evaluated for a particular health rule.

    * `RUN_DURATION_SECONDS`: Expected total time for a run in seconds.
    * `STREAMING_BACKLOG_BYTES`: An estimate of the maximum bytes of data waiting to be consumed across all streams. This metric is in Public Preview.
    * `STREAMING_BACKLOG_RECORDS`: An estimate of the maximum offset lag across all streams. This metric is in Public Preview.
    * `STREAMING_BACKLOG_SECONDS`: An estimate of the maximum consumer delay across all streams. This metric is in Public Preview.
    * `STREAMING_BACKLOG_FILES`: An estimate of the maximum number of outstanding files across all streams. This metric is in Public Preview.
    """

    RUN_DURATION_SECONDS = "RUN_DURATION_SECONDS"
    STREAMING_BACKLOG_BYTES = "STREAMING_BACKLOG_BYTES"
    STREAMING_BACKLOG_RECORDS = "STREAMING_BACKLOG_RECORDS"
    STREAMING_BACKLOG_SECONDS = "STREAMING_BACKLOG_SECONDS"
    STREAMING_BACKLOG_FILES = "STREAMING_BACKLOG_FILES"


JobsHealthMetricParam = (
    Literal[
        "RUN_DURATION_SECONDS",
        "STREAMING_BACKLOG_BYTES",
        "STREAMING_BACKLOG_RECORDS",
        "STREAMING_BACKLOG_SECONDS",
        "STREAMING_BACKLOG_FILES",
    ]
    | JobsHealthMetric
)



from dataclasses import dataclass
from typing import Type

from databricks.bundles.core._resource import Resource


@dataclass(kw_only=True, frozen=True)
class _ResourceType:
    """
    NB: this class should stay internal-only and NOT be exported from databricks.bundles.core
    """

    resource_type: Type[Resource]

    singular_name: str
    """
    Singular name, should be used in methods (e.g. "add_job"), error messages and as parameter names.
    """

    plural_name: str
    """
    Plural name, the same as in "resources" bundle section.
    """

    @classmethod
    def all(cls) -> tuple["_ResourceType", ...]:
        """
        Returns all supported resource types.
        """

        # intentionally lazily load all resource types to avoid imports from databricks.bundles.core to
        # be imported in databricks.bundles.<resource_type>

        from databricks.bundles.jobs._models.job import Job
        from databricks.bundles.pipelines._models.pipeline import Pipeline
        from databricks.bundles.schemas._models.schema import Schema
        from databricks.bundles.volumes._models.volume import Volume

        return (
            _ResourceType(
                resource_type=Job,
                singular_name="job",
                plural_name="jobs",
            ),
            _ResourceType(
                resource_type=Pipeline,
                plural_name="pipelines",
                singular_name="pipeline",
            ),
            _ResourceType(
                resource_type=Volume,
                plural_name="volumes",
                singular_name="volume",
            ),
            _ResourceType(
                resource_type=Schema,
                plural_name="schemas",
                singular_name="schema",
            ),
        )



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import (
    VariableOr,
    VariableOrList,
    VariableOrOptional,
)
from databricks.bundles.jobs._models.sql_task_subscription import (
    SqlTaskSubscription,
    SqlTaskSubscriptionParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SqlTaskAlert:
    """"""

    alert_id: VariableOr[str]
    """
    The canonical identifier of the SQL alert.
    """

    pause_subscriptions: VariableOrOptional[bool] = None
    """
    If true, the alert notifications are not sent to subscribers.
    """

    subscriptions: VariableOrList[SqlTaskSubscription] = field(default_factory=list)
    """
    If specified, alert notifications are sent to subscribers.
    """

    @classmethod
    def from_dict(cls, value: "SqlTaskAlertDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SqlTaskAlertDict":
        return _transform_to_json_value(self)  # type:ignore


class SqlTaskAlertDict(TypedDict, total=False):
    """"""

    alert_id: VariableOr[str]
    """
    The canonical identifier of the SQL alert.
    """

    pause_subscriptions: VariableOrOptional[bool]
    """
    If true, the alert notifications are not sent to subscribers.
    """

    subscriptions: VariableOrList[SqlTaskSubscriptionParam]
    """
    If specified, alert notifications are sent to subscribers.
    """


SqlTaskAlertParam = SqlTaskAlertDict | SqlTaskAlert



__all__ = [
    "Lifecycle",
    "LifecycleDict",
    "LifecycleParam",
    "Volume",
    "VolumeDict",
    "VolumeGrant",
    "VolumeGrantDict",
    "VolumeGrantParam",
    "VolumeGrantPrivilege",
    "VolumeGrantPrivilegeParam",
    "VolumeParam",
    "VolumeType",
    "VolumeTypeParam",
]


from databricks.bundles.volumes._models.lifecycle import (
    Lifecycle,
    LifecycleDict,
    LifecycleParam,
)
from databricks.bundles.volumes._models.volume import Volume, VolumeDict, VolumeParam
from databricks.bundles.volumes._models.volume_grant import (
    VolumeGrant,
    VolumeGrantDict,
    VolumeGrantParam,
)
from databricks.bundles.volumes._models.volume_grant_privilege import (
    VolumeGrantPrivilege,
    VolumeGrantPrivilegeParam,
)
from databricks.bundles.volumes._models.volume_type import VolumeType, VolumeTypeParam



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr
from databricks.bundles.jobs._models.periodic_trigger_configuration_time_unit import (
    PeriodicTriggerConfigurationTimeUnit,
    PeriodicTriggerConfigurationTimeUnitParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PeriodicTriggerConfiguration:
    """"""

    interval: VariableOr[int]
    """
    The interval at which the trigger should run.
    """

    unit: VariableOr[PeriodicTriggerConfigurationTimeUnit]
    """
    The unit of time for the interval.
    """

    @classmethod
    def from_dict(cls, value: "PeriodicTriggerConfigurationDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PeriodicTriggerConfigurationDict":
        return _transform_to_json_value(self)  # type:ignore


class PeriodicTriggerConfigurationDict(TypedDict, total=False):
    """"""

    interval: VariableOr[int]
    """
    The interval at which the trigger should run.
    """

    unit: VariableOr[PeriodicTriggerConfigurationTimeUnitParam]
    """
    The unit of time for the interval.
    """


PeriodicTriggerConfigurationParam = (
    PeriodicTriggerConfigurationDict | PeriodicTriggerConfiguration
)



from enum import Enum
from typing import Literal


class AwsAvailability(Enum):
    """
    Availability type used for all subsequent nodes past the `first_on_demand` ones.

    Note: If `first_on_demand` is zero, this availability type will be used for the entire cluster.
    """

    SPOT = "SPOT"
    ON_DEMAND = "ON_DEMAND"
    SPOT_WITH_FALLBACK = "SPOT_WITH_FALLBACK"


AwsAvailabilityParam = (
    Literal["SPOT", "ON_DEMAND", "SPOT_WITH_FALLBACK"] | AwsAvailability
)



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class PipelinesEnvironment:
    """
    The environment entity used to preserve serverless environment side panel, jobs' environment for non-notebook task, and DLT's environment for classic and serverless pipelines.
    In this minimal environment spec, only pip dependencies are supported.
    """

    dependencies: VariableOrList[str] = field(default_factory=list)
    """
    List of pip dependencies, as supported by the version of pip in this environment.
    Each dependency is a pip requirement file line https://pip.pypa.io/en/stable/reference/requirements-file-format/
    Allowed dependency could be <requirement specifier>, <archive url/path>, <local project path>(WSFS or Volumes in Databricks), <vcs project url>
    """

    @classmethod
    def from_dict(cls, value: "PipelinesEnvironmentDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "PipelinesEnvironmentDict":
        return _transform_to_json_value(self)  # type:ignore


class PipelinesEnvironmentDict(TypedDict, total=False):
    """"""

    dependencies: VariableOrList[str]
    """
    List of pip dependencies, as supported by the version of pip in this environment.
    Each dependency is a pip requirement file line https://pip.pypa.io/en/stable/reference/requirements-file-format/
    Allowed dependency could be <requirement specifier>, <archive url/path>, <local project path>(WSFS or Volumes in Databricks), <vcs project url>
    """


PipelinesEnvironmentParam = PipelinesEnvironmentDict | PipelinesEnvironment



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.pipelines._models.azure_availability import (
    AzureAvailability,
    AzureAvailabilityParam,
)
from databricks.bundles.pipelines._models.log_analytics_info import (
    LogAnalyticsInfo,
    LogAnalyticsInfoParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class AzureAttributes:
    """
    Attributes set during cluster creation which are related to Microsoft Azure.
    """

    availability: VariableOrOptional[AzureAvailability] = None

    first_on_demand: VariableOrOptional[int] = None
    """
    The first `first_on_demand` nodes of the cluster will be placed on on-demand instances.
    This value should be greater than 0, to make sure the cluster driver node is placed on an
    on-demand instance. If this value is greater than or equal to the current cluster size, all
    nodes will be placed on on-demand instances. If this value is less than the current cluster
    size, `first_on_demand` nodes will be placed on on-demand instances and the remainder will
    be placed on `availability` instances. Note that this value does not affect
    cluster size and cannot currently be mutated over the lifetime of a cluster.
    """

    log_analytics_info: VariableOrOptional[LogAnalyticsInfo] = None
    """
    Defines values necessary to configure and run Azure Log Analytics agent
    """

    spot_bid_max_price: VariableOrOptional[float] = None
    """
    The max bid price to be used for Azure spot instances.
    The Max price for the bid cannot be higher than the on-demand price of the instance.
    If not specified, the default value is -1, which specifies that the instance cannot be evicted
    on the basis of price, and only on the basis of availability. Further, the value should > 0 or -1.
    """

    @classmethod
    def from_dict(cls, value: "AzureAttributesDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "AzureAttributesDict":
        return _transform_to_json_value(self)  # type:ignore


class AzureAttributesDict(TypedDict, total=False):
    """"""

    availability: VariableOrOptional[AzureAvailabilityParam]

    first_on_demand: VariableOrOptional[int]
    """
    The first `first_on_demand` nodes of the cluster will be placed on on-demand instances.
    This value should be greater than 0, to make sure the cluster driver node is placed on an
    on-demand instance. If this value is greater than or equal to the current cluster size, all
    nodes will be placed on on-demand instances. If this value is less than the current cluster
    size, `first_on_demand` nodes will be placed on on-demand instances and the remainder will
    be placed on `availability` instances. Note that this value does not affect
    cluster size and cannot currently be mutated over the lifetime of a cluster.
    """

    log_analytics_info: VariableOrOptional[LogAnalyticsInfoParam]
    """
    Defines values necessary to configure and run Azure Log Analytics agent
    """

    spot_bid_max_price: VariableOrOptional[float]
    """
    The max bid price to be used for Azure spot instances.
    The Max price for the bid cannot be higher than the on-demand price of the instance.
    If not specified, the default value is -1, which specifies that the instance cannot be evicted
    on the basis of price, and only on the basis of availability. Further, the value should > 0 or -1.
    """


AzureAttributesParam = AzureAttributesDict | AzureAttributes



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class Notifications:
    """"""

    alerts: VariableOrList[str] = field(default_factory=list)
    """
    A list of alerts that trigger the sending of notifications to the configured
    destinations. The supported alerts are:
    
    * `on-update-success`: A pipeline update completes successfully.
    * `on-update-failure`: Each time a pipeline update fails.
    * `on-update-fatal-failure`: A pipeline update fails with a non-retryable (fatal) error.
    * `on-flow-failure`: A single data flow fails.
    """

    email_recipients: VariableOrList[str] = field(default_factory=list)
    """
    A list of email addresses notified when a configured alert is triggered.
    """

    @classmethod
    def from_dict(cls, value: "NotificationsDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "NotificationsDict":
        return _transform_to_json_value(self)  # type:ignore


class NotificationsDict(TypedDict, total=False):
    """"""

    alerts: VariableOrList[str]
    """
    A list of alerts that trigger the sending of notifications to the configured
    destinations. The supported alerts are:
    
    * `on-update-success`: A pipeline update completes successfully.
    * `on-update-failure`: Each time a pipeline update fails.
    * `on-update-fatal-failure`: A pipeline update fails with a non-retryable (fatal) error.
    * `on-flow-failure`: A single data flow fails.
    """

    email_recipients: VariableOrList[str]
    """
    A list of email addresses notified when a configured alert is triggered.
    """


NotificationsParam = NotificationsDict | Notifications



from enum import Enum
from typing import Literal


class PeriodicTriggerConfigurationTimeUnit(Enum):
    HOURS = "HOURS"
    DAYS = "DAYS"
    WEEKS = "WEEKS"


PeriodicTriggerConfigurationTimeUnitParam = (
    Literal["HOURS", "DAYS", "WEEKS"] | PeriodicTriggerConfigurationTimeUnit
)



from enum import Enum
from typing import Literal


class GitProvider(Enum):
    GIT_HUB = "gitHub"
    BITBUCKET_CLOUD = "bitbucketCloud"
    AZURE_DEV_OPS_SERVICES = "azureDevOpsServices"
    GIT_HUB_ENTERPRISE = "gitHubEnterprise"
    BITBUCKET_SERVER = "bitbucketServer"
    GIT_LAB = "gitLab"
    GIT_LAB_ENTERPRISE_EDITION = "gitLabEnterpriseEdition"
    AWS_CODE_COMMIT = "awsCodeCommit"


GitProviderParam = (
    Literal[
        "gitHub",
        "bitbucketCloud",
        "azureDevOpsServices",
        "gitHubEnterprise",
        "bitbucketServer",
        "gitLab",
        "gitLabEnterpriseEdition",
        "awsCodeCommit",
    ]
    | GitProvider
)



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList, VariableOrOptional
from databricks.bundles.jobs._models.condition import Condition, ConditionParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class TableUpdateTriggerConfiguration:
    """"""

    condition: VariableOrOptional[Condition] = None
    """
    The table(s) condition based on which to trigger a job run.
    """

    min_time_between_triggers_seconds: VariableOrOptional[int] = None
    """
    If set, the trigger starts a run only after the specified amount of time has passed since
    the last time the trigger fired. The minimum allowed value is 60 seconds.
    """

    table_names: VariableOrList[str] = field(default_factory=list)
    """
    A list of tables to monitor for changes. The table name must be in the format `catalog_name.schema_name.table_name`.
    """

    wait_after_last_change_seconds: VariableOrOptional[int] = None
    """
    If set, the trigger starts a run only after no table updates have occurred for the specified time
    and can be used to wait for a series of table updates before triggering a run. The
    minimum allowed value is 60 seconds.
    """

    @classmethod
    def from_dict(cls, value: "TableUpdateTriggerConfigurationDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "TableUpdateTriggerConfigurationDict":
        return _transform_to_json_value(self)  # type:ignore


class TableUpdateTriggerConfigurationDict(TypedDict, total=False):
    """"""

    condition: VariableOrOptional[ConditionParam]
    """
    The table(s) condition based on which to trigger a job run.
    """

    min_time_between_triggers_seconds: VariableOrOptional[int]
    """
    If set, the trigger starts a run only after the specified amount of time has passed since
    the last time the trigger fired. The minimum allowed value is 60 seconds.
    """

    table_names: VariableOrList[str]
    """
    A list of tables to monitor for changes. The table name must be in the format `catalog_name.schema_name.table_name`.
    """

    wait_after_last_change_seconds: VariableOrOptional[int]
    """
    If set, the trigger starts a run only after no table updates have occurred for the specified time
    and can be used to wait for a series of table updates before triggering a run. The
    minimum allowed value is 60 seconds.
    """


TableUpdateTriggerConfigurationParam = (
    TableUpdateTriggerConfigurationDict | TableUpdateTriggerConfiguration
)



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList
from databricks.bundles.jobs._models.webhook import Webhook, WebhookParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class WebhookNotifications:
    """"""

    on_duration_warning_threshold_exceeded: VariableOrList[Webhook] = field(
        default_factory=list
    )
    """
    An optional list of system notification IDs to call when the duration of a run exceeds the threshold specified for the `RUN_DURATION_SECONDS` metric in the `health` field. A maximum of 3 destinations can be specified for the `on_duration_warning_threshold_exceeded` property.
    """

    on_failure: VariableOrList[Webhook] = field(default_factory=list)
    """
    An optional list of system notification IDs to call when the run fails. A maximum of 3 destinations can be specified for the `on_failure` property.
    """

    on_start: VariableOrList[Webhook] = field(default_factory=list)
    """
    An optional list of system notification IDs to call when the run starts. A maximum of 3 destinations can be specified for the `on_start` property.
    """

    on_streaming_backlog_exceeded: VariableOrList[Webhook] = field(default_factory=list)
    """
    An optional list of system notification IDs to call when any streaming backlog thresholds are exceeded for any stream.
    Streaming backlog thresholds can be set in the `health` field using the following metrics: `STREAMING_BACKLOG_BYTES`, `STREAMING_BACKLOG_RECORDS`, `STREAMING_BACKLOG_SECONDS`, or `STREAMING_BACKLOG_FILES`.
    Alerting is based on the 10-minute average of these metrics. If the issue persists, notifications are resent every 30 minutes.
    A maximum of 3 destinations can be specified for the `on_streaming_backlog_exceeded` property.
    """

    on_success: VariableOrList[Webhook] = field(default_factory=list)
    """
    An optional list of system notification IDs to call when the run completes successfully. A maximum of 3 destinations can be specified for the `on_success` property.
    """

    @classmethod
    def from_dict(cls, value: "WebhookNotificationsDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "WebhookNotificationsDict":
        return _transform_to_json_value(self)  # type:ignore


class WebhookNotificationsDict(TypedDict, total=False):
    """"""

    on_duration_warning_threshold_exceeded: VariableOrList[WebhookParam]
    """
    An optional list of system notification IDs to call when the duration of a run exceeds the threshold specified for the `RUN_DURATION_SECONDS` metric in the `health` field. A maximum of 3 destinations can be specified for the `on_duration_warning_threshold_exceeded` property.
    """

    on_failure: VariableOrList[WebhookParam]
    """
    An optional list of system notification IDs to call when the run fails. A maximum of 3 destinations can be specified for the `on_failure` property.
    """

    on_start: VariableOrList[WebhookParam]
    """
    An optional list of system notification IDs to call when the run starts. A maximum of 3 destinations can be specified for the `on_start` property.
    """

    on_streaming_backlog_exceeded: VariableOrList[WebhookParam]
    """
    An optional list of system notification IDs to call when any streaming backlog thresholds are exceeded for any stream.
    Streaming backlog thresholds can be set in the `health` field using the following metrics: `STREAMING_BACKLOG_BYTES`, `STREAMING_BACKLOG_RECORDS`, `STREAMING_BACKLOG_SECONDS`, or `STREAMING_BACKLOG_FILES`.
    Alerting is based on the 10-minute average of these metrics. If the issue persists, notifications are resent every 30 minutes.
    A maximum of 3 destinations can be specified for the `on_streaming_backlog_exceeded` property.
    """

    on_success: VariableOrList[WebhookParam]
    """
    An optional list of system notification IDs to call when the run completes successfully. A maximum of 3 destinations can be specified for the `on_success` property.
    """


WebhookNotificationsParam = WebhookNotificationsDict | WebhookNotifications



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOr, VariableOrOptional
from databricks.bundles.jobs._models.environment import Environment, EnvironmentParam

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class JobEnvironment:
    """"""

    environment_key: VariableOr[str]
    """
    The key of an environment. It has to be unique within a job.
    """

    spec: VariableOrOptional[Environment] = None

    @classmethod
    def from_dict(cls, value: "JobEnvironmentDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "JobEnvironmentDict":
        return _transform_to_json_value(self)  # type:ignore


class JobEnvironmentDict(TypedDict, total=False):
    """"""

    environment_key: VariableOr[str]
    """
    The key of an environment. It has to be unique within a job.
    """

    spec: VariableOrOptional[EnvironmentParam]


JobEnvironmentParam = JobEnvironmentDict | JobEnvironment



from enum import Enum
from typing import Literal


class DayOfWeek(Enum):
    """
    :meta private: [EXPERIMENTAL]

    Days of week in which the window is allowed to happen.
    If not specified all days of the week will be used.
    """

    MONDAY = "MONDAY"
    TUESDAY = "TUESDAY"
    WEDNESDAY = "WEDNESDAY"
    THURSDAY = "THURSDAY"
    FRIDAY = "FRIDAY"
    SATURDAY = "SATURDAY"
    SUNDAY = "SUNDAY"


DayOfWeekParam = (
    Literal[
        "MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY", "SUNDAY"
    ]
    | DayOfWeek
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrOptional
from databricks.bundles.pipelines._models.gcp_availability import (
    GcpAvailability,
    GcpAvailabilityParam,
)

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class GcpAttributes:
    """
    Attributes set during cluster creation which are related to GCP.
    """

    availability: VariableOrOptional[GcpAvailability] = None

    boot_disk_size: VariableOrOptional[int] = None
    """
    Boot disk size in GB
    """

    first_on_demand: VariableOrOptional[int] = None
    """
    The first `first_on_demand` nodes of the cluster will be placed on on-demand instances.
    This value should be greater than 0, to make sure the cluster driver node is placed on an
    on-demand instance. If this value is greater than or equal to the current cluster size, all
    nodes will be placed on on-demand instances. If this value is less than the current cluster
    size, `first_on_demand` nodes will be placed on on-demand instances and the remainder will
    be placed on `availability` instances. Note that this value does not affect
    cluster size and cannot currently be mutated over the lifetime of a cluster.
    """

    google_service_account: VariableOrOptional[str] = None
    """
    If provided, the cluster will impersonate the google service account when accessing
    gcloud services (like GCS). The google service account
    must have previously been added to the Databricks environment by an account
    administrator.
    """

    local_ssd_count: VariableOrOptional[int] = None
    """
    If provided, each node (workers and driver) in the cluster will have this number of local SSDs attached.
    Each local SSD is 375GB in size.
    Refer to [GCP documentation](https://cloud.google.com/compute/docs/disks/local-ssd#choose_number_local_ssds)
    for the supported number of local SSDs for each instance type.
    """

    use_preemptible_executors: VariableOrOptional[bool] = None
    """
    [DEPRECATED] This field determines whether the spark executors will be scheduled to run on preemptible
    VMs (when set to true) versus standard compute engine VMs (when set to false; default).
    Note: Soon to be deprecated, use the 'availability' field instead.
    """

    zone_id: VariableOrOptional[str] = None
    """
    Identifier for the availability zone in which the cluster resides.
    This can be one of the following:
    - "HA" => High availability, spread nodes across availability zones for a Databricks deployment region [default].
    - "AUTO" => Databricks picks an availability zone to schedule the cluster on.
    - A GCP availability zone => Pick One of the available zones for (machine type + region) from
    https://cloud.google.com/compute/docs/regions-zones.
    """

    @classmethod
    def from_dict(cls, value: "GcpAttributesDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "GcpAttributesDict":
        return _transform_to_json_value(self)  # type:ignore


class GcpAttributesDict(TypedDict, total=False):
    """"""

    availability: VariableOrOptional[GcpAvailabilityParam]

    boot_disk_size: VariableOrOptional[int]
    """
    Boot disk size in GB
    """

    first_on_demand: VariableOrOptional[int]
    """
    The first `first_on_demand` nodes of the cluster will be placed on on-demand instances.
    This value should be greater than 0, to make sure the cluster driver node is placed on an
    on-demand instance. If this value is greater than or equal to the current cluster size, all
    nodes will be placed on on-demand instances. If this value is less than the current cluster
    size, `first_on_demand` nodes will be placed on on-demand instances and the remainder will
    be placed on `availability` instances. Note that this value does not affect
    cluster size and cannot currently be mutated over the lifetime of a cluster.
    """

    google_service_account: VariableOrOptional[str]
    """
    If provided, the cluster will impersonate the google service account when accessing
    gcloud services (like GCS). The google service account
    must have previously been added to the Databricks environment by an account
    administrator.
    """

    local_ssd_count: VariableOrOptional[int]
    """
    If provided, each node (workers and driver) in the cluster will have this number of local SSDs attached.
    Each local SSD is 375GB in size.
    Refer to [GCP documentation](https://cloud.google.com/compute/docs/disks/local-ssd#choose_number_local_ssds)
    for the supported number of local SSDs for each instance type.
    """

    use_preemptible_executors: VariableOrOptional[bool]
    """
    [DEPRECATED] This field determines whether the spark executors will be scheduled to run on preemptible
    VMs (when set to true) versus standard compute engine VMs (when set to false; default).
    Note: Soon to be deprecated, use the 'availability' field instead.
    """

    zone_id: VariableOrOptional[str]
    """
    Identifier for the availability zone in which the cluster resides.
    This can be one of the following:
    - "HA" => High availability, spread nodes across availability zones for a Databricks deployment region [default].
    - "AUTO" => Databricks picks an availability zone to schedule the cluster on.
    - A GCP availability zone => Pick One of the available zones for (machine type + region) from
    https://cloud.google.com/compute/docs/regions-zones.
    """


GcpAttributesParam = GcpAttributesDict | GcpAttributes



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from databricks.bundles.core._transform import _transform
from databricks.bundles.core._transform_to_json import _transform_to_json_value
from databricks.bundles.core._variable import VariableOrList

if TYPE_CHECKING:
    from typing_extensions import Self


@dataclass(kw_only=True)
class SparkSubmitTask:
    """
    [DEPRECATED]
    """

    parameters: VariableOrList[str] = field(default_factory=list)
    """
    Command-line parameters passed to spark submit.
    
    Use [Task parameter variables](https://docs.databricks.com/jobs.html#parameter-variables) to set parameters containing information about job runs.
    """

    @classmethod
    def from_dict(cls, value: "SparkSubmitTaskDict") -> "Self":
        return _transform(cls, value)

    def as_dict(self) -> "SparkSubmitTaskDict":
        return _transform_to_json_value(self)  # type:ignore


class SparkSubmitTaskDict(TypedDict, total=False):
    """"""

    parameters: VariableOrList[str]
    """
    Command-line parameters passed to spark submit.
    
    Use [Task parameter variables](https://docs.databricks.com/jobs.html#parameter-variables) to set parameters containing information about job runs.
    """


SparkSubmitTaskParam = SparkSubmitTaskDict | SparkSubmitTask
